@@ -126,7 +126,7 @@ public class Log4j1ConfigurationParser {
     }
 
     private void buildProperties() {
-        for (Map.Entry<Object, Object> entry : new TreeMap<>(properties).entrySet()) {
+        for (final Map.Entry<Object, Object> entry : new TreeMap<>(properties).entrySet()) {
             final String key = entry.getKey().toString();
             if (!key.startsWith("log4j.") && !key.equals(ROOTCATEGORY) && !key.equals(ROOTLOGGER)) {
                 builder.addProperty(key, Objects.toString(entry.getValue(), Strings.EMPTY));
@@ -134,7 +134,7 @@ public class Log4j1ConfigurationParser {
         }
     }
 
-    private void warn(String string) {
+    private void warn(final String string) {
         System.err.println(string);
     }
 
@@ -280,7 +280,7 @@ public class Log4j1ConfigurationParser {
         }
     }
 
-    private void buildNullAppender(String appenderName) {
+    private void buildNullAppender(final String appenderName) {
         final AppenderComponentBuilder appenderBuilder = builder.newAppender(appenderName, NullAppender.PLUGIN_NAME);
         builder.add(appenderBuilder);
     }
@@ -403,7 +403,7 @@ public class Log4j1ConfigurationParser {
                                 // Add Appenders to this logger
                                 final String[] sortedAppenderNames = Arrays.copyOfRange(split, 1, split.length);
                                 Arrays.sort(sortedAppenderNames);
-                                for (String appenderName : sortedAppenderNames) {
+                                for (final String appenderName : sortedAppenderNames) {
                                     newLogger.add(builder.newAppenderRef(appenderName));
                                 }
                             }
@@ -425,7 +425,7 @@ public class Log4j1ConfigurationParser {
         return strSubstitutorProperties.replace(sysValue);
     }
 
-    private String getProperty(final String key, String defaultValue) {
+    private String getProperty(final String key, final String defaultValue) {
         final String value = getProperty(key);
         return value == null ? defaultValue : value;
     }
