@@ -32,13 +32,13 @@ import org.apache.logging.log4j.status.StatusLogger;
 import org.apache.logging.log4j.util.LoaderUtil;
 import org.apache.logging.log4j.util.PropertiesUtil;
 import org.apache.logging.log4j.util.ProviderUtil;
-import org.apache.logging.log4j.util.ReflectionUtil;
+import org.apache.logging.log4j.util.StackLocatorUtil;
 import org.apache.logging.log4j.util.Strings;
 
 /**
- * The anchor point for the logging system. The most common usage of this class is to obtain a named {@link Logger}. The
- * method {@link #getLogger()} is provided as the most convenient way to obtain a named Logger based on the calling
- * class name. This class also provides method for obtaining named Loggers that use
+ * The anchor point for the Log4j logging system. The most common usage of this class is to obtain a named
+ * {@link Logger}. The method {@link #getLogger()} is provided as the most convenient way to obtain a named Logger based
+ * on the calling class name. This class also provides method for obtaining named Loggers that use
  * {@link String#format(String, Object...)} style messages instead of the default type of parameterized messages. These
  * are obtained through the {@link #getFormatterLogger(Class)} family of methods. Other service provider methods are
  * given through the {@link #getContext()} and {@link #getFactory()} family of methods; these methods are not normally
@@ -93,8 +93,8 @@ public class LogManager {
                         try {
                             factories.put(provider.getPriority(), factoryClass.newInstance());
                         } catch (final Exception e) {
-                            LOGGER.error("Unable to create class {} specified in {}", factoryClass.getName(), provider
-                                    .getUrl().toString(), e);
+                            LOGGER.error("Unable to create class {} specified in provider URL {}", factoryClass.getName(), provider
+                                    .getUrl(), e);
                         }
                     }
                 }
@@ -333,6 +333,31 @@ public class LogManager {
         }
     }
 
+
+    /**
+     * Returns a LoggerContext
+     *
+     * @param fqcn The fully qualified class name of the Class that this method is a member of.
+     * @param loader The ClassLoader for the context. If null the context will attempt to determine the appropriate
+     *            ClassLoader.
+     * @param currentContext if false the LoggerContext appropriate for the caller of this method is returned. For
+     *            example, in a web application if the caller is a class in WEB-INF/lib then one LoggerContext may be
+     *            returned and if the caller is a class in the container's classpath then a different LoggerContext may
+     *            be returned. If true then only a single LoggerContext will be returned.
+     * @param configLocation The URI for the configuration to use.
+     * @param name The LoggerContext name.
+     * @return a LoggerContext.
+     */
+    protected static LoggerContext getContext(final String fqcn, final ClassLoader loader,
+                                              final boolean currentContext, URI configLocation, String name) {
+        try {
+            return factory.getContext(fqcn, loader, null, currentContext, configLocation, name);
+        } catch (final IllegalStateException ex) {
+            LOGGER.warn(ex.getMessage() + " Using SimpleLogger");
+            return new SimpleLoggerContextFactory().getContext(fqcn, loader, null, currentContext);
+        }
+    }
+
     /**
      * Shutdown using the LoggerContext appropriate for the caller of this method.
      * This is equivalent to calling {@code LogManager.shutdown(false)}.
@@ -380,6 +405,11 @@ public class LogManager {
         }
     }
 
+    private static String toLoggerName(final Class<?> cls) {
+        String canonicalName = cls.getCanonicalName();
+        return canonicalName != null ? canonicalName : cls.getName();
+    }
+
     /**
      * Returns the current LoggerContextFactory.
      *
@@ -418,7 +448,7 @@ public class LogManager {
      * @since 2.4
      */
     public static Logger getFormatterLogger() {
-        return getFormatterLogger(ReflectionUtil.getCallerClass(2));
+        return getFormatterLogger(StackLocatorUtil.getCallerClass(2));
     }
 
     /**
@@ -449,7 +479,7 @@ public class LogManager {
      * @see StringFormatterMessageFactory
      */
     public static Logger getFormatterLogger(final Class<?> clazz) {
-        return getLogger(clazz != null ? clazz : ReflectionUtil.getCallerClass(2),
+        return getLogger(clazz != null ? clazz : StackLocatorUtil.getCallerClass(2),
                 StringFormatterMessageFactory.INSTANCE);
     }
 
@@ -481,7 +511,7 @@ public class LogManager {
      * @see StringFormatterMessageFactory
      */
     public static Logger getFormatterLogger(final Object value) {
-        return getLogger(value != null ? value.getClass() : ReflectionUtil.getCallerClass(2),
+        return getLogger(value != null ? value.getClass() : StackLocatorUtil.getCallerClass(2),
                 StringFormatterMessageFactory.INSTANCE);
     }
 
@@ -512,7 +542,7 @@ public class LogManager {
      * @see StringFormatterMessageFactory
      */
     public static Logger getFormatterLogger(final String name) {
-        return name == null ? getFormatterLogger(ReflectionUtil.getCallerClass(2)) : getLogger(name,
+        return name == null ? getFormatterLogger(StackLocatorUtil.getCallerClass(2)) : getLogger(name,
                 StringFormatterMessageFactory.INSTANCE);
     }
 
@@ -520,7 +550,7 @@ public class LogManager {
         if (clazz != null) {
             return clazz;
         }
-        final Class<?> candidate = ReflectionUtil.getCallerClass(3);
+        final Class<?> candidate = StackLocatorUtil.getCallerClass(3);
         if (candidate == null) {
             throw new UnsupportedOperationException("No class provided, and an appropriate one cannot be found.");
         }
@@ -534,7 +564,7 @@ public class LogManager {
      * @throws UnsupportedOperationException if the calling class cannot be determined.
      */
     public static Logger getLogger() {
-        return getLogger(ReflectionUtil.getCallerClass(2));
+        return getLogger(StackLocatorUtil.getCallerClass(2));
     }
 
     /**
@@ -548,7 +578,7 @@ public class LogManager {
      */
     public static Logger getLogger(final Class<?> clazz) {
         final Class<?> cls = callerClass(clazz);
-        return getContext(cls.getClassLoader(), false).getLogger(cls.getName());
+        return getContext(cls.getClassLoader(), false).getLogger(toLoggerName(cls));
     }
 
     /**
@@ -564,7 +594,7 @@ public class LogManager {
      */
     public static Logger getLogger(final Class<?> clazz, final MessageFactory messageFactory) {
         final Class<?> cls = callerClass(clazz);
-        return getContext(cls.getClassLoader(), false).getLogger(cls.getName(), messageFactory);
+        return getContext(cls.getClassLoader(), false).getLogger(toLoggerName(cls), messageFactory);
     }
 
     /**
@@ -576,7 +606,7 @@ public class LogManager {
      * @throws UnsupportedOperationException if the calling class cannot be determined.
      */
     public static Logger getLogger(final MessageFactory messageFactory) {
-        return getLogger(ReflectionUtil.getCallerClass(2), messageFactory);
+        return getLogger(StackLocatorUtil.getCallerClass(2), messageFactory);
     }
 
     /**
@@ -589,7 +619,7 @@ public class LogManager {
      *             determined.
      */
     public static Logger getLogger(final Object value) {
-        return getLogger(value != null ? value.getClass() : ReflectionUtil.getCallerClass(2));
+        return getLogger(value != null ? value.getClass() : StackLocatorUtil.getCallerClass(2));
     }
 
     /**
@@ -604,7 +634,7 @@ public class LogManager {
      *             determined.
      */
     public static Logger getLogger(final Object value, final MessageFactory messageFactory) {
-        return getLogger(value != null ? value.getClass() : ReflectionUtil.getCallerClass(2), messageFactory);
+        return getLogger(value != null ? value.getClass() : StackLocatorUtil.getCallerClass(2), messageFactory);
     }
 
     /**
@@ -615,7 +645,7 @@ public class LogManager {
      * @throws UnsupportedOperationException if {@code name} is {@code null} and the calling class cannot be determined.
      */
     public static Logger getLogger(final String name) {
-        return name != null ? getContext(false).getLogger(name) : getLogger(ReflectionUtil.getCallerClass(2));
+        return name != null ? getContext(false).getLogger(name) : getLogger(StackLocatorUtil.getCallerClass(2));
     }
 
     /**
@@ -629,7 +659,7 @@ public class LogManager {
      */
     public static Logger getLogger(final String name, final MessageFactory messageFactory) {
         return name != null ? getContext(false).getLogger(name, messageFactory) : getLogger(
-                ReflectionUtil.getCallerClass(2), messageFactory);
+                StackLocatorUtil.getCallerClass(2), messageFactory);
     }
 
     /**
