@@ -25,6 +25,7 @@ import java.util.Map;
 import java.util.Set;
 
 import org.apache.logging.log4j.util.StringBuilderFormattable;
+import org.apache.logging.log4j.util.StringBuilders;
 
 /**
  * Supports parameter formatting as used in ParameterizedMessage and ReusableParameterizedMessage.
@@ -391,12 +392,36 @@ final class ParameterFormatter {
         if (o == null) {
             return null;
         }
+        // Check special types to avoid unnecessary StringBuilder usage
         if (o instanceof String) {
             return (String) o;
         }
+        if (o instanceof Integer) {
+            return Integer.toString((Integer) o);
+        }
+        if (o instanceof Long) {
+            return Long.toString((Long) o);
+        }
+        if (o instanceof Double) {
+            return Double.toString((Double) o);
+        }
+        if (o instanceof Boolean) {
+            return Boolean.toString((Boolean) o);
+        }
+        if (o instanceof Character) {
+            return Character.toString((Character) o);
+        }
+        if (o instanceof Short) {
+            return Short.toString((Short) o);
+        }
+        if (o instanceof Float) {
+            return Float.toString((Float) o);
+        }
+        if (o instanceof Byte) {
+            return Byte.toString((Byte) o);
+        }
         final StringBuilder str = new StringBuilder();
-        final Set<String> dejaVu = new HashSet<>(); // that's actually a neat name ;)
-        recursiveDeepToString(o, str, dejaVu);
+        recursiveDeepToString(o, str, null);
         return str.toString();
     }
 
@@ -424,7 +449,7 @@ final class ParameterFormatter {
      * @param str    the StringBuilder that o will be appended to
      * @param dejaVu a list of container identities that were already used.
      */
-    private static void recursiveDeepToString(final Object o, final StringBuilder str, final Set<String> dejaVu) {
+    static void recursiveDeepToString(final Object o, final StringBuilder str, final Set<String> dejaVu) {
         if (appendSpecialTypes(o, str)) {
             return;
         }
@@ -436,38 +461,7 @@ final class ParameterFormatter {
     }
 
     private static boolean appendSpecialTypes(final Object o, final StringBuilder str) {
-        if (o == null || o instanceof String) {
-            str.append((String) o);
-            return true;
-        } else if (o instanceof CharSequence) {
-            str.append((CharSequence) o);
-            return true;
-        } else if (o instanceof StringBuilderFormattable) {
-            ((StringBuilderFormattable) o).formatTo(str);
-            return true;
-        } else if (o instanceof Integer) { // LOG4J2-1415 unbox auto-boxed primitives to avoid calling toString()
-            str.append(((Integer) o).intValue());
-            return true;
-        } else if (o instanceof Long) {
-            str.append(((Long) o).longValue());
-            return true;
-        } else if (o instanceof Double) {
-            str.append(((Double) o).doubleValue());
-            return true;
-        } else if (o instanceof Boolean) {
-            str.append(((Boolean) o).booleanValue());
-            return true;
-        } else if (o instanceof Character) {
-            str.append(((Character) o).charValue());
-            return true;
-        } else if (o instanceof Short) {
-            str.append(((Short) o).shortValue());
-            return true;
-        } else if (o instanceof Float) {
-            str.append(((Float) o).floatValue());
-            return true;
-        }
-        return appendDate(o, str);
+        return StringBuilders.appendSpecificTypes(str, o) || appendDate(o, str);
     }
 
     private static boolean appendDate(final Object o, final StringBuilder str) {
