@@ -1,4 +1,4 @@
-package org.apache.logging.log4j.util;/*
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements. See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
@@ -14,13 +14,21 @@ package org.apache.logging.log4j.util;/*
  * See the license for the specific language governing permissions and
  * limitations under the license.
  */
+package org.apache.logging.log4j.util;
 
+import java.io.BufferedReader;
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
+import java.io.File;
+import java.io.FileOutputStream;
 import java.io.IOException;
+import java.io.InputStreamReader;
 import java.io.ObjectInputStream;
 import java.io.ObjectOutputStream;
 import java.lang.reflect.Field;
+import java.net.URL;
+import java.net.URLDecoder;
+import java.nio.charset.Charset;
 import java.util.ConcurrentModificationException;
 import java.util.HashMap;
 import java.util.Map;
@@ -46,7 +54,7 @@ public class SortedArrayStringMapTest {
 
     @Test
     public void testConstructorIgnoresNull() throws Exception {
-        assertEquals(0, new SortedArrayStringMap(null).size());
+        assertEquals(0, new SortedArrayStringMap((SortedArrayStringMap) null).size());
     }
 
     @Test
@@ -70,6 +78,86 @@ public class SortedArrayStringMapTest {
         assertEquals(original, copy);
     }
 
+    @Test
+    public void testSerializationOfNonSerializableValue() throws Exception {
+        final SortedArrayStringMap original = new SortedArrayStringMap();
+        original.putValue("a", "avalue");
+        original.putValue("B", "Bvalue");
+        original.putValue("unserializable", new Object());
+
+        final byte[] binary = serialize(original);
+        final SortedArrayStringMap copy = deserialize(binary);
+
+        final SortedArrayStringMap expected = new SortedArrayStringMap();
+        expected.putValue("a", "avalue");
+        expected.putValue("B", "Bvalue");
+        expected.putValue("unserializable", null);
+        assertEquals(expected, copy);
+    }
+
+    @Test
+    public void testDeserializationOfUnknownClass() throws Exception {
+        final SortedArrayStringMap original = new SortedArrayStringMap();
+        original.putValue("a", "avalue");
+        original.putValue("serializableButNotInClasspathOfDeserializer", new org.junit.runner.Result());
+        original.putValue("zz", "last");
+
+        final File file = new File("target/SortedArrayStringMap.ser");
+        try (FileOutputStream fout = new FileOutputStream(file, false)) {
+            fout.write(serialize(original));
+            fout.flush();
+        }
+        final String classpath = createClassPath(SortedArrayStringMap.class, DeserializerHelper.class);
+        final Process process = new ProcessBuilder("java", "-cp", classpath,
+                DeserializerHelper.class.getName(), file.getPath()).start();
+        final BufferedReader in = new BufferedReader(new InputStreamReader(process.getErrorStream()));
+        final int exitValue = process.waitFor();
+
+        file.delete();
+        if (exitValue != 0) {
+            final StringBuilder sb = new StringBuilder();
+            sb.append("DeserializerHelper exited with error code ").append(exitValue);
+            sb.append(". Classpath='").append(classpath);
+            sb.append("'. Process output: ");
+            int c = -1;
+            while ((c = in.read()) != -1) {
+                sb.append((char) c);
+            }
+            fail(sb.toString());
+        }
+    }
+
+    private String createClassPath(final Class<?>... classes) throws Exception {
+        final StringBuilder result = new StringBuilder();
+        for (final Class<?> cls : classes) {
+            if (result.length() > 0) {
+                result.append(File.pathSeparator);
+            }
+            result.append(createClassPath(cls));
+        }
+        return result.toString();
+    }
+
+    private String createClassPath(final Class<?> cls) throws Exception {
+        final String resource = "/" + cls.getName().replace('.', '/') + ".class";
+        final URL url = cls.getResource(resource);
+        String location = url.toString();
+        if (location.startsWith("jar:")) {
+            location = location.substring("jar:".length(), location.indexOf('!'));
+        }
+        if (location.startsWith("file:/")) {
+            location = location.substring("file:/".length());
+        }
+        if (location.endsWith(resource)) {
+            location = location.substring(0, location.length() - resource.length());
+        }
+        if (!new File(location).exists()) {
+            location = File.separator + location;
+        }
+        location = URLDecoder.decode(location, Charset.defaultCharset().name()); // replace %20 with ' ' etc
+        return location.isEmpty() ? "." : location;
+    }
+
     private byte[] serialize(final SortedArrayStringMap data) throws IOException {
         final ByteArrayOutputStream arr = new ByteArrayOutputStream();
         final ObjectOutputStream out = new ObjectOutputStream(arr);
