@@ -26,6 +26,7 @@ import java.util.concurrent.TimeUnit;
 import org.apache.logging.log4j.core.Layout;
 import org.apache.logging.log4j.core.LoggerContext;
 import org.apache.logging.log4j.core.layout.ByteBufferDestination;
+import org.apache.logging.log4j.core.layout.ByteBufferDestinationHelper;
 import org.apache.logging.log4j.core.util.Constants;
 
 /**
@@ -35,7 +36,7 @@ import org.apache.logging.log4j.core.util.Constants;
 public class OutputStreamManager extends AbstractManager implements ByteBufferDestination {
     protected final Layout<?> layout;
     protected ByteBuffer byteBuffer;
-    private volatile OutputStream os;
+    private volatile OutputStream outputStream;
     private boolean skipFooter;
 
     protected OutputStreamManager(final OutputStream os, final String streamName, final Layout<?> layout,
@@ -58,7 +59,7 @@ public class OutputStreamManager extends AbstractManager implements ByteBufferDe
     protected OutputStreamManager(final OutputStream os, final String streamName, final Layout<?> layout,
             final boolean writeHeader, final ByteBuffer byteBuffer) {
         super(null, streamName);
-        this.os = os;
+        this.outputStream = os;
         this.layout = layout;
         if (writeHeader && layout != null) {
             final byte[] header = layout.getHeader();
@@ -87,7 +88,7 @@ public class OutputStreamManager extends AbstractManager implements ByteBufferDe
         }
         this.layout = layout;
         this.byteBuffer = Objects.requireNonNull(byteBuffer, "byteBuffer");
-        this.os = os;
+        this.outputStream = os;
         if (writeHeader && layout != null) {
             final byte[] header = layout.getHeader();
             if (header != null) {
@@ -118,7 +119,7 @@ public class OutputStreamManager extends AbstractManager implements ByteBufferDe
     protected OutputStream createOutputStream() throws IOException {
         throw new IllegalStateException(getClass().getCanonicalName() + " must implement createOutputStream()");
     }
-    
+
     /**
      * Indicate whether the footer should be skipped or not.
      * @param skipFooter true if the footer should be skipped.
@@ -157,11 +158,15 @@ public class OutputStreamManager extends AbstractManager implements ByteBufferDe
         return getCount() > 0;
     }
 
+    public boolean hasOutputStream() {
+        return outputStream != null;
+    }
+
     protected OutputStream getOutputStream() throws IOException {
-        if (os == null) {
-            os = createOutputStream();
+        if (outputStream == null) {
+            outputStream = createOutputStream();
         }
-        return os;
+        return outputStream;
     }
 
     protected void setOutputStream(final OutputStream os) {
@@ -169,12 +174,12 @@ public class OutputStreamManager extends AbstractManager implements ByteBufferDe
         if (header != null) {
             try {
                 os.write(header, 0, header.length);
-                this.os = os; // only update field if os.write() succeeded
+                this.outputStream = os; // only update field if os.write() succeeded
             } catch (final IOException ioe) {
                 logError("Unable to write header", ioe);
             }
         } else {
-            this.os = os;
+            this.outputStream = os;
         }
     }
 
@@ -197,6 +202,11 @@ public class OutputStreamManager extends AbstractManager implements ByteBufferDe
         write(bytes, 0, bytes.length, immediateFlush);
     }
 
+    @Override
+    public void writeBytes(final byte[] data, final int offset, final int length) {
+        write(data, offset, length, false);
+    }
+
     /**
      * Some output streams synchronize writes while others do not. Synchronizing here insures that
      * log events won't be intertwined.
@@ -206,7 +216,7 @@ public class OutputStreamManager extends AbstractManager implements ByteBufferDe
      * @throws AppenderLoggingException if an error occurs.
      */
     protected void write(final byte[] bytes, final int offset, final int length) {
-        write(bytes, offset, length, false);
+        writeBytes(bytes, offset, length);
     }
 
     /**
@@ -260,7 +270,7 @@ public class OutputStreamManager extends AbstractManager implements ByteBufferDe
      * @since 2.6
      */
     protected synchronized void flushDestination() {
-        final OutputStream stream = os; // access volatile field only once per method
+        final OutputStream stream = outputStream; // access volatile field only once per method
         if (stream != null) {
             try {
                 stream.flush();
@@ -280,8 +290,8 @@ public class OutputStreamManager extends AbstractManager implements ByteBufferDe
      */
     protected synchronized void flushBuffer(final ByteBuffer buf) {
         buf.flip();
-        if (buf.limit() > 0) {
-            writeToDestination(buf.array(), 0, buf.limit());
+        if (buf.remaining() > 0) {
+            writeToDestination(buf.array(), buf.arrayOffset() + buf.position(), buf.remaining());
         }
         buf.clear();
     }
@@ -296,7 +306,7 @@ public class OutputStreamManager extends AbstractManager implements ByteBufferDe
 
     protected synchronized boolean closeOutputStream() {
         flush();
-        final OutputStream stream = os; // access volatile field only once per method
+        final OutputStream stream = outputStream; // access volatile field only once per method
         if (stream == null || stream == System.out || stream == System.err) {
             return true;
         }
@@ -341,4 +351,14 @@ public class OutputStreamManager extends AbstractManager implements ByteBufferDe
         flushBuffer(buf);
         return buf;
     }
+
+    @Override
+    public void writeBytes(final ByteBuffer data) {
+        if (data.remaining() == 0) {
+          return;
+        }
+        synchronized (this) {
+          ByteBufferDestinationHelper.writeToUnsynchronized(data, this);
+        }
+    }
 }
