@@ -23,10 +23,13 @@ import java.util.concurrent.TimeUnit;
 import java.util.zip.Deflater;
 
 import org.apache.logging.log4j.core.Appender;
+import org.apache.logging.log4j.core.Core;
 import org.apache.logging.log4j.core.Filter;
 import org.apache.logging.log4j.core.Layout;
 import org.apache.logging.log4j.core.LogEvent;
 import org.apache.logging.log4j.core.appender.rolling.DefaultRolloverStrategy;
+import org.apache.logging.log4j.core.appender.rolling.DirectFileRolloverStrategy;
+import org.apache.logging.log4j.core.appender.rolling.DirectWriteRolloverStrategy;
 import org.apache.logging.log4j.core.appender.rolling.RollingFileManager;
 import org.apache.logging.log4j.core.appender.rolling.RolloverStrategy;
 import org.apache.logging.log4j.core.appender.rolling.TriggeringPolicy;
@@ -34,7 +37,6 @@ import org.apache.logging.log4j.core.config.Configuration;
 import org.apache.logging.log4j.core.config.plugins.Plugin;
 import org.apache.logging.log4j.core.config.plugins.PluginBuilderAttribute;
 import org.apache.logging.log4j.core.config.plugins.PluginBuilderFactory;
-import org.apache.logging.log4j.core.config.plugins.PluginConfiguration;
 import org.apache.logging.log4j.core.config.plugins.PluginElement;
 import org.apache.logging.log4j.core.config.plugins.validation.constraints.Required;
 import org.apache.logging.log4j.core.net.Advertiser;
@@ -44,7 +46,7 @@ import org.apache.logging.log4j.core.util.Integers;
 /**
  * An appender that writes to files and can roll over at intervals.
  */
-@Plugin(name = RollingFileAppender.PLUGIN_NAME, category = "Core", elementType = Appender.ELEMENT_TYPE, printObject = true)
+@Plugin(name = RollingFileAppender.PLUGIN_NAME, category = Core.CATEGORY_NAME, elementType = Appender.ELEMENT_TYPE, printObject = true)
 public final class RollingFileAppender extends AbstractOutputStreamAppender<RollingFileManager> {
 
     public static final String PLUGIN_NAME = "RollingFile";
@@ -53,13 +55,12 @@ public final class RollingFileAppender extends AbstractOutputStreamAppender<Roll
      * Builds FileAppender instances.
      * 
      * @param <B>
-     *            This builder class
+     *            The type to build
      */
     public static class Builder<B extends Builder<B>> extends AbstractOutputStreamAppender.Builder<B>
             implements org.apache.logging.log4j.core.util.Builder<RollingFileAppender> {
 
         @PluginBuilderAttribute
-        @Required
         private String fileName;
 
         @PluginBuilderAttribute
@@ -88,8 +89,14 @@ public final class RollingFileAppender extends AbstractOutputStreamAppender<Roll
         @PluginBuilderAttribute
         private boolean createOnDemand;
 
-        @PluginConfiguration
-        private Configuration configuration;
+        @PluginBuilderAttribute
+        private String filePermissions;
+
+        @PluginBuilderAttribute
+        private String fileOwner;
+
+        @PluginBuilderAttribute
+        private String fileGroup;
 
         @Override
         public RollingFileAppender build() {
@@ -106,11 +113,6 @@ public final class RollingFileAppender extends AbstractOutputStreamAppender<Roll
                 LOGGER.warn("RollingFileAppender '{}': The bufferSize is set to {} but bufferedIO is not true", getName(), bufferSize);
             }
 
-            if (fileName == null) {
-                LOGGER.error("RollingFileAppender '{}': No file name provided.", getName());
-                return null;
-            }
-
             if (filePattern == null) {
                 LOGGER.error("RollingFileAppender '{}': No file name pattern provided.", getName());
                 return null;
@@ -122,19 +124,26 @@ public final class RollingFileAppender extends AbstractOutputStreamAppender<Roll
             }
 
             if (strategy == null) {
-                strategy = DefaultRolloverStrategy.createStrategy(null, null, null,
-                        String.valueOf(Deflater.DEFAULT_COMPRESSION), null, true, configuration);
-            }
-
-            if (strategy == null) {
-                strategy = DefaultRolloverStrategy.createStrategy(null, null, null,
-                        String.valueOf(Deflater.DEFAULT_COMPRESSION), null, true, configuration);
+                if (fileName != null) {
+                    strategy = DefaultRolloverStrategy.newBuilder()
+                                        .withCompressionLevelStr(String.valueOf(Deflater.DEFAULT_COMPRESSION))
+                                        .withConfig(getConfiguration())
+                                        .build();
+                } else {
+                    strategy = DirectWriteRolloverStrategy.newBuilder()
+                                        .withCompressionLevelStr(String.valueOf(Deflater.DEFAULT_COMPRESSION))
+                                        .withConfig(getConfiguration())
+                                        .build();
+                }
+            } else if (fileName == null && !(strategy instanceof DirectFileRolloverStrategy)) {
+                LOGGER.error("RollingFileAppender '{}': When no file name is provided a DirectFilenameRolloverStrategy must be configured");
+                return null;
             }
 
             final Layout<? extends Serializable> layout = getOrCreateLayout();
             final RollingFileManager manager = RollingFileManager.getFileManager(fileName, filePattern, append,
                     isBufferedIo, policy, strategy, advertiseUri, layout, bufferSize, isImmediateFlush(),
-                    createOnDemand, configuration);
+                    createOnDemand, filePermissions, fileOwner, fileGroup, getConfiguration());
             if (manager == null) {
                 return null;
             }
@@ -142,17 +151,13 @@ public final class RollingFileAppender extends AbstractOutputStreamAppender<Roll
             manager.initialize();
 
             return new RollingFileAppender(getName(), layout, getFilter(), manager, fileName, filePattern,
-                    isIgnoreExceptions(), isImmediateFlush(), advertise ? configuration.getAdvertiser() : null);
+                    isIgnoreExceptions(), isImmediateFlush(), advertise ? getConfiguration().getAdvertiser() : null);
         }
 
         public String getAdvertiseUri() {
             return advertiseUri;
         }
 
-        public Configuration getConfiguration() {
-            return configuration;
-        }
-
         public String getFileName() {
             return fileName;
         }
@@ -173,6 +178,18 @@ public final class RollingFileAppender extends AbstractOutputStreamAppender<Roll
             return locking;
         }
 
+        public String getFilePermissions() {
+            return filePermissions;
+        }
+
+        public String getFileOwner() {
+            return fileOwner;
+        }
+
+        public String getFileGroup() {
+            return fileGroup;
+        }
+
         public B withAdvertise(final boolean advertise) {
             this.advertise = advertise;
             return asBuilder();
@@ -188,11 +205,6 @@ public final class RollingFileAppender extends AbstractOutputStreamAppender<Roll
             return asBuilder();
         }
 
-        public B withConfiguration(final Configuration config) {
-            this.configuration = config;
-            return asBuilder();
-        }
-
         public B withFileName(final String fileName) {
             this.fileName = fileName;
             return asBuilder();
@@ -235,6 +247,21 @@ public final class RollingFileAppender extends AbstractOutputStreamAppender<Roll
             return asBuilder();
         }
 
+        public B withFilePermissions(final String filePermissions) {
+            this.filePermissions = filePermissions;
+            return asBuilder();
+        }
+
+        public B withFileOwner(final String fileOwner) {
+            this.fileOwner = fileOwner;
+            return asBuilder();
+        }
+
+        public B withFileGroup(final String fileGroup) {
+            this.fileGroup = fileGroup;
+            return asBuilder();
+        }
+
     }
     
     private static final int DEFAULT_BUFFER_SIZE = 8192;
@@ -329,7 +356,7 @@ public final class RollingFileAppender extends AbstractOutputStreamAppender<Roll
      * @deprecated Use {@link #newBuilder()}.
      */
     @Deprecated
-    public static RollingFileAppender createAppender(
+    public static <B extends Builder<B>> RollingFileAppender createAppender(
             // @formatter:off
             final String fileName,
             final String filePattern,
@@ -349,13 +376,13 @@ public final class RollingFileAppender extends AbstractOutputStreamAppender<Roll
             // @formatter:on
         final int bufferSize = Integers.parseInt(bufferSizeStr, DEFAULT_BUFFER_SIZE);
         // @formatter:off
-        return newBuilder()
+        return RollingFileAppender.<B>newBuilder()
                 .withAdvertise(Boolean.parseBoolean(advertise))
                 .withAdvertiseUri(advertiseUri)
                 .withAppend(Booleans.parseBoolean(append, true))
                 .withBufferedIo(Booleans.parseBoolean(bufferedIO, true))
                 .withBufferSize(bufferSize)
-                .withConfiguration(config)
+                .setConfiguration(config)
                 .withFileName(fileName)
                 .withFilePattern(filePattern)
                 .withFilter(filter)
