@@ -18,6 +18,7 @@ package org.apache.logging.log4j.core.appender.rolling;
 
 import java.lang.reflect.Method;
 
+import org.apache.logging.log4j.core.Core;
 import org.apache.logging.log4j.core.LogEvent;
 import org.apache.logging.log4j.core.config.plugins.Plugin;
 import org.apache.logging.log4j.core.config.plugins.PluginAttribute;
@@ -28,7 +29,7 @@ import org.apache.logging.log4j.status.StatusLogger;
 /**
  * Triggers a rollover on every restart, but only if the file size is greater than zero.
  */
-@Plugin(name = "OnStartupTriggeringPolicy", category = "Core", printObject = true)
+@Plugin(name = "OnStartupTriggeringPolicy", category = Core.CATEGORY_NAME, printObject = true)
 public class OnStartupTriggeringPolicy extends AbstractTriggeringPolicy {
 
     private static final long JVM_START_TIME = initStartTime();
