@@ -22,8 +22,11 @@ import java.io.IOException;
 import java.io.OutputStream;
 import java.io.Serializable;
 import java.nio.ByteBuffer;
-import java.util.concurrent.Future;
+import java.util.Collection;
+import java.util.concurrent.ArrayBlockingQueue;
+import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Semaphore;
+import java.util.concurrent.ThreadPoolExecutor;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicReferenceFieldUpdater;
 
@@ -39,6 +42,8 @@ import org.apache.logging.log4j.core.appender.rolling.action.AbstractAction;
 import org.apache.logging.log4j.core.appender.rolling.action.Action;
 import org.apache.logging.log4j.core.config.Configuration;
 import org.apache.logging.log4j.core.util.Constants;
+import org.apache.logging.log4j.core.util.FileUtils;
+import org.apache.logging.log4j.core.util.Log4jThreadFactory;
 
 /**
  * The Rolling File Manager.
@@ -46,14 +51,25 @@ import org.apache.logging.log4j.core.util.Constants;
 public class RollingFileManager extends FileManager {
 
     private static RollingFileManagerFactory factory = new RollingFileManagerFactory();
+    private static final int MAX_TRIES = 3;
+    private static final int MIN_DURATION = 100;
 
     protected long size;
     private long initialTime;
-    private final PatternProcessor patternProcessor;
+    private volatile PatternProcessor patternProcessor;
     private final Semaphore semaphore = new Semaphore(1);
+    private final Log4jThreadFactory threadFactory = Log4jThreadFactory.createThreadFactory("RollingFileManager");
     private volatile TriggeringPolicy triggeringPolicy;
     private volatile RolloverStrategy rolloverStrategy;
     private volatile boolean renameEmptyFiles = false;
+    private volatile boolean initialized = false;
+    private volatile String fileName;
+    private final FileExtension fileExtension;
+
+    /* This executor pool will create a new Thread for every work async action to be performed. Using it allows
+       us to make sure all the Threads are completed when the Manager is stopped. */
+    private final ExecutorService asyncExecutor = new ThreadPoolExecutor(0, Integer.MAX_VALUE, 0, TimeUnit.MILLISECONDS,
+            new EmptyQueue(), threadFactory);
 
     private static final AtomicReferenceFieldUpdater<RollingFileManager, TriggeringPolicy> triggeringPolicyUpdater =
             AtomicReferenceFieldUpdater.newUpdater(RollingFileManager.class, TriggeringPolicy.class, "triggeringPolicy");
@@ -61,6 +77,9 @@ public class RollingFileManager extends FileManager {
     private static final AtomicReferenceFieldUpdater<RollingFileManager, RolloverStrategy> rolloverStrategyUpdater =
             AtomicReferenceFieldUpdater.newUpdater(RollingFileManager.class, RolloverStrategy.class, "rolloverStrategy");
 
+    private static final AtomicReferenceFieldUpdater<RollingFileManager, PatternProcessor> patternProcessorUpdater =
+            AtomicReferenceFieldUpdater.newUpdater(RollingFileManager.class, PatternProcessor.class, "patternProcessor");
+
     @Deprecated
     protected RollingFileManager(final String fileName, final String pattern, final OutputStream os,
             final boolean append, final long size, final long time, final TriggeringPolicy triggeringPolicy,
@@ -82,11 +101,11 @@ public class RollingFileManager extends FileManager {
         this.rolloverStrategy = rolloverStrategy;
         this.patternProcessor = new PatternProcessor(pattern);
         this.patternProcessor.setPrevFileTime(time);
+        this.fileName = fileName;
+        this.fileExtension = FileExtension.lookupForFile(pattern);
     }
 
-    /**
-     * @since 2.7
-     */
+    @Deprecated
     protected RollingFileManager(final LoggerContext loggerContext, final String fileName, final String pattern, final OutputStream os,
             final boolean append, final boolean createOnDemand, final long size, final long time,
             final TriggeringPolicy triggeringPolicy, final RolloverStrategy rolloverStrategy,
@@ -98,10 +117,41 @@ public class RollingFileManager extends FileManager {
         this.rolloverStrategy = rolloverStrategy;
         this.patternProcessor = new PatternProcessor(pattern);
         this.patternProcessor.setPrevFileTime(time);
+        this.fileName = fileName;
+        this.fileExtension = FileExtension.lookupForFile(pattern);
+    }
+
+    /**
+     * @since 2.9
+     */
+    protected RollingFileManager(final LoggerContext loggerContext, final String fileName, final String pattern, final OutputStream os,
+            final boolean append, final boolean createOnDemand, final long size, final long time,
+            final TriggeringPolicy triggeringPolicy, final RolloverStrategy rolloverStrategy,
+            final String advertiseURI, final Layout<? extends Serializable> layout,
+            final String filePermissions, final String fileOwner, final String fileGroup,
+            final boolean writeHeader, final ByteBuffer buffer) {
+        super(loggerContext, fileName, os, append, false, createOnDemand, advertiseURI, layout,
+              filePermissions, fileOwner, fileGroup, writeHeader, buffer);
+        this.size = size;
+        this.initialTime = time;
+        this.triggeringPolicy = triggeringPolicy;
+        this.rolloverStrategy = rolloverStrategy;
+        this.patternProcessor = new PatternProcessor(pattern);
+        this.patternProcessor.setPrevFileTime(time);
+        this.fileName = fileName;
+        this.fileExtension = FileExtension.lookupForFile(pattern);
     }
 
     public void initialize() {
-        triggeringPolicy.initialize(this);
+
+        if (!initialized) {
+            LOGGER.debug("Initializing triggering policy {}", triggeringPolicy);
+            initialized = true;
+            triggeringPolicy.initialize(this);
+            if (triggeringPolicy instanceof LifeCycle) {
+                ((LifeCycle) triggeringPolicy).start();
+            }
+        }
     }
 
     /**
@@ -117,16 +167,43 @@ public class RollingFileManager extends FileManager {
      * @param bufferSize buffer size to use if bufferedIO is true
      * @param immediateFlush flush on every write or not
      * @param createOnDemand true if you want to lazy-create the file (a.k.a. on-demand.)
+     * @param filePermissions File permissions
+     * @param fileOwner File owner
+     * @param fileGroup File group
      * @param configuration The configuration.
      * @return A RollingFileManager.
      */
     public static RollingFileManager getFileManager(final String fileName, final String pattern, final boolean append,
             final boolean bufferedIO, final TriggeringPolicy policy, final RolloverStrategy strategy,
             final String advertiseURI, final Layout<? extends Serializable> layout, final int bufferSize,
-            final boolean immediateFlush, final boolean createOnDemand, final Configuration configuration) {
+            final boolean immediateFlush, final boolean createOnDemand,
+            final String filePermissions, final String fileOwner, final String fileGroup,
+            final Configuration configuration) {
 
-        return (RollingFileManager) getManager(fileName, new FactoryData(pattern, append,
-            bufferedIO, policy, strategy, advertiseURI, layout, bufferSize, immediateFlush, createOnDemand, configuration), factory);
+        if (strategy instanceof DirectWriteRolloverStrategy && fileName != null) {
+            LOGGER.error("The fileName attribute must not be specified with the DirectWriteRolloverStrategy");
+            return null;
+        }
+        final String name = fileName == null ? pattern : fileName;
+        return narrow(RollingFileManager.class, getManager(name, new FactoryData(fileName, pattern, append,
+            bufferedIO, policy, strategy, advertiseURI, layout, bufferSize, immediateFlush, createOnDemand,
+            filePermissions, fileOwner, fileGroup, configuration), factory));
+    }
+
+    /**
+     * Returns the name of the File being managed.
+     * @return The name of the File being managed.
+     */
+    @Override
+    public String getFileName() {
+        if (rolloverStrategy instanceof DirectFileRolloverStrategy) {
+            fileName = ((DirectFileRolloverStrategy) rolloverStrategy).getCurrentFileName(this);
+        }
+        return fileName;
+    }
+
+    public FileExtension getFileExtension() {
+        return fileExtension;
     }
 
     // override to make visible for unit tests
@@ -178,6 +255,7 @@ public class RollingFileManager extends FileManager {
 
     @Override
     public boolean releaseSub(final long timeout, final TimeUnit timeUnit) {
+        LOGGER.debug("Shutting down RollingFileManager {}" + getName());
         boolean stopped = true;
         if (triggeringPolicy instanceof LifeCycle2) {
             stopped &= ((LifeCycle2) triggeringPolicy).stop(timeout, timeUnit);
@@ -185,10 +263,52 @@ public class RollingFileManager extends FileManager {
             ((LifeCycle) triggeringPolicy).stop();
             stopped &= true;
         }
-        return stopped && super.releaseSub(timeout, timeUnit);
+        final boolean status = super.releaseSub(timeout, timeUnit) && stopped;
+        asyncExecutor.shutdown();
+        try {
+            // Allow at least the minimum interval to pass so async actions can complete.
+            final long millis = timeUnit.toMillis(timeout);
+            final long waitInterval = MIN_DURATION < millis ? millis : MIN_DURATION;
+
+            for (int count = 1; count <= MAX_TRIES && !asyncExecutor.isTerminated(); ++count) {
+                asyncExecutor.awaitTermination(waitInterval * count, TimeUnit.MILLISECONDS);
+            }
+            if (asyncExecutor.isTerminated()) {
+                LOGGER.debug("All asynchronous threads have terminated");
+            } else {
+                asyncExecutor.shutdownNow();
+                try {
+                    asyncExecutor.awaitTermination(timeout, timeUnit);
+                    if (asyncExecutor.isTerminated()) {
+                        LOGGER.debug("All asynchronous threads have terminated");
+                    } else {
+                        LOGGER.debug("RollingFileManager shutting down but some asynchronous services may not have completed");
+                    }
+                } catch (final InterruptedException inner) {
+                    LOGGER.warn("RollingFileManager stopped but some asynchronous services may not have completed.");
+                }
+            }
+        } catch (final InterruptedException ie) {
+            asyncExecutor.shutdownNow();
+            try {
+                asyncExecutor.awaitTermination(timeout, timeUnit);
+                if (asyncExecutor.isTerminated()) {
+                    LOGGER.debug("All asynchronous threads have terminated");
+                }
+            } catch (final InterruptedException inner) {
+                LOGGER.warn("RollingFileManager stopped but some asynchronous services may not have completed.");
+            }
+            // Preserve interrupt status
+            Thread.currentThread().interrupt();
+        }
+        LOGGER.debug("RollingFileManager shutdown completed with status {}", status);
+        return status;
     }
 
     public synchronized void rollover() {
+        if (!hasOutputStream()) {
+            return;
+        }
         if (rollover(rolloverStrategy)) {
             try {
                 size = 0;
@@ -201,7 +321,7 @@ public class RollingFileManager extends FileManager {
     }
 
     protected void createFileAfterRollover() throws IOException  {
-        setOutputStream(new FileOutputStream(getFileName(), isAppend()));
+        setOutputStream(createOutputStream());
     }
 
     /**
@@ -214,13 +334,35 @@ public class RollingFileManager extends FileManager {
 
     public void setTriggeringPolicy(final TriggeringPolicy triggeringPolicy) {
         triggeringPolicy.initialize(this);
-        triggeringPolicyUpdater.compareAndSet(this, this.triggeringPolicy, triggeringPolicy);
+        final TriggeringPolicy policy = this.triggeringPolicy;
+        int count = 0;
+        boolean policyUpdated = false;
+        do {
+            ++count;
+        } while (!(policyUpdated = triggeringPolicyUpdater.compareAndSet(this, this.triggeringPolicy, triggeringPolicy))
+                && count < MAX_TRIES);
+        if (policyUpdated) {
+            if (triggeringPolicy instanceof LifeCycle) {
+                ((LifeCycle) triggeringPolicy).start();
+            }
+            if (policy instanceof LifeCycle) {
+                ((LifeCycle) policy).stop();
+            }
+        } else {
+            if (triggeringPolicy instanceof LifeCycle) {
+                ((LifeCycle) triggeringPolicy).stop();
+            }
+        }
     }
 
     public void setRolloverStrategy(final RolloverStrategy rolloverStrategy) {
         rolloverStrategyUpdater.compareAndSet(this, this.rolloverStrategy, rolloverStrategy);
     }
 
+    public void setPatternProcessor(final PatternProcessor patternProcessor) {
+        patternProcessorUpdater.compareAndSet(this, this.patternProcessor, patternProcessor);
+    }
+
     /**
      * Returns the triggering policy.
      * @param <T> TriggeringPolicy type
@@ -242,16 +384,17 @@ public class RollingFileManager extends FileManager {
 
     private boolean rollover(final RolloverStrategy strategy) {
 
+        boolean releaseRequired = false;
         try {
             // Block until the asynchronous operation is completed.
             semaphore.acquire();
+            releaseRequired = true;
         } catch (final InterruptedException e) {
             logError("Thread interrupted while attempting to check rollover", e);
             return false;
         }
 
-        boolean success = false;
-        Future<?> future = null;
+        boolean success = true;
 
         try {
             final RolloverDescription descriptor = strategy.rollover(this);
@@ -263,19 +406,21 @@ public class RollingFileManager extends FileManager {
                     try {
                         success = descriptor.getSynchronous().execute();
                     } catch (final Exception ex) {
+                        success = false;
                         logError("Caught error in synchronous task", ex);
                     }
                 }
 
                 if (success && descriptor.getAsynchronous() != null) {
                     LOGGER.debug("RollingFileManager executing async {}", descriptor.getAsynchronous());
-                    future = LoggerContext.getContext(false).submit(new AsyncAction(descriptor.getAsynchronous(), this));
+                    asyncExecutor.execute(new AsyncAction(descriptor.getAsynchronous(), this));
+                    releaseRequired = false;
                 }
                 return true;
             }
             return false;
         } finally {
-            if (future == null || future.isDone() || future.isCancelled()) {
+            if (releaseRequired) {
                 semaphore.release();
             }
         }
@@ -356,6 +501,7 @@ public class RollingFileManager extends FileManager {
      * Factory data.
      */
     private static class FactoryData extends ConfigurationFactoryData {
+        private final String fileName;
         private final String pattern;
         private final boolean append;
         private final boolean bufferedIO;
@@ -366,6 +512,9 @@ public class RollingFileManager extends FileManager {
         private final RolloverStrategy strategy;
         private final String advertiseURI;
         private final Layout<? extends Serializable> layout;
+        private final String filePermissions;
+        private final String fileOwner;
+        private final String fileGroup;
 
         /**
          * Creates the data for the factory.
@@ -377,13 +526,18 @@ public class RollingFileManager extends FileManager {
          * @param bufferSize the buffer size
          * @param immediateFlush flush on every write or not
          * @param createOnDemand true if you want to lazy-create the file (a.k.a. on-demand.)
+         * @param filePermissions File permissions
+         * @param fileOwner File owner
+         * @param fileGroup File group
          * @param configuration The configuration
          */
-        public FactoryData(final String pattern, final boolean append, final boolean bufferedIO,
+        public FactoryData(final String fileName, final String pattern, final boolean append, final boolean bufferedIO,
                 final TriggeringPolicy policy, final RolloverStrategy strategy, final String advertiseURI,
                 final Layout<? extends Serializable> layout, final int bufferSize, final boolean immediateFlush,
-                final boolean createOnDemand, final Configuration configuration) {
+                final boolean createOnDemand, final String filePermissions, final String fileOwner, final String fileGroup,
+                final Configuration configuration) {
             super(configuration);
+            this.fileName = fileName;
             this.pattern = pattern;
             this.append = append;
             this.bufferedIO = bufferedIO;
@@ -394,18 +548,23 @@ public class RollingFileManager extends FileManager {
             this.layout = layout;
             this.immediateFlush = immediateFlush;
             this.createOnDemand = createOnDemand;
+            this.filePermissions = filePermissions;
+            this.fileOwner = fileOwner;
+            this.fileGroup = fileGroup;
         }
 
-        public TriggeringPolicy getTriggeringPolicy()
-        {
+        public TriggeringPolicy getTriggeringPolicy() {
             return this.policy;
         }
 
-        public RolloverStrategy getRolloverStrategy()
-        {
+        public RolloverStrategy getRolloverStrategy() {
             return this.strategy;
         }
 
+        public String getPattern() {
+            return pattern;
+        }
+
         @Override
         public String toString() {
             final StringBuilder builder = new StringBuilder();
@@ -426,17 +585,21 @@ public class RollingFileManager extends FileManager {
             builder.append(advertiseURI);
             builder.append(", layout=");
             builder.append(layout);
+            builder.append(", filePermissions=");
+            builder.append(filePermissions);
+            builder.append(", fileOwner=");
+            builder.append(fileOwner);
             builder.append("]");
             return builder.toString();
         }
     }
 
     @Override
-    public void updateData(final Object data)
-    {
+    public void updateData(final Object data) {
         final FactoryData factoryData = (FactoryData) data;
         setRolloverStrategy(factoryData.getRolloverStrategy());
         setTriggeringPolicy(factoryData.getTriggeringPolicy());
+        setPatternProcessor(new PatternProcessor(factoryData.getPattern(), getPatternProcessor()));
     }
 
     /**
@@ -452,31 +615,41 @@ public class RollingFileManager extends FileManager {
          */
         @Override
         public RollingFileManager createManager(final String name, final FactoryData data) {
-            final File file = new File(name);
-            final File parent = file.getParentFile();
-            if (null != parent && !parent.exists()) {
-                parent.mkdirs();
+            long size = 0;
+            boolean writeHeader = !data.append;
+            File file = null;
+            if (data.fileName != null) {
+                file = new File(data.fileName);
+                // LOG4J2-1140: check writeHeader before creating the file
+                writeHeader = !data.append || !file.exists();
+
+                try {
+                    FileUtils.makeParentDirs(file);
+                    final boolean created = data.createOnDemand ? false : file.createNewFile();
+                    LOGGER.trace("New file '{}' created = {}", name, created);
+                } catch (final IOException ioe) {
+                    LOGGER.error("Unable to create file " + name, ioe);
+                    return null;
+                }
+                size = data.append ? file.length() : 0;
             }
-            // LOG4J2-1140: check writeHeader before creating the file
-            final boolean writeHeader = !data.append || !file.exists();
-            try {
-                final boolean created = data.createOnDemand ? false : file.createNewFile();
-                LOGGER.trace("New file '{}' created = {}", name, created);
-            } catch (final IOException ioe) {
-                LOGGER.error("Unable to create file " + name, ioe);
-                return null;
-            }
-            final long size = data.append ? file.length() : 0;
 
             try {
                 final int actualSize = data.bufferedIO ? data.bufferSize : Constants.ENCODER_BYTE_BUFFER_SIZE;
                 final ByteBuffer buffer = ByteBuffer.wrap(new byte[actualSize]);
-                final OutputStream os = data.createOnDemand ? null : new FileOutputStream(name, data.append);
-                final long time = data.createOnDemand? System.currentTimeMillis() : file.lastModified(); // LOG4J2-531 create file first so time has valid value
+                final OutputStream os = data.createOnDemand  || data.fileName == null ? null :
+                        new FileOutputStream(data.fileName, data.append);
+                final long time = data.createOnDemand || file == null ?
+                        System.currentTimeMillis() : file.lastModified(); // LOG4J2-531 create file first so time has valid value
 
-                return new RollingFileManager(data.getLoggerContext(), name, data.pattern, os,
-                        data.append, data.createOnDemand, size, time, data.policy, data.strategy, data.advertiseURI,
-                        data.layout, writeHeader, buffer);
+                final RollingFileManager rm = new RollingFileManager(data.getLoggerContext(), data.fileName, data.pattern, os,
+                    data.append, data.createOnDemand, size, time, data.policy, data.strategy, data.advertiseURI,
+                    data.layout, data.filePermissions, data.fileOwner, data.fileGroup, writeHeader, buffer);
+                if (os != null && rm.isAttributeViewEnabled()) {
+                    rm.defineAttributeView(file.toPath());
+                }
+
+                return rm;
             } catch (final IOException ex) {
                 LOGGER.error("RollingFileManager (" + name + ") " + ex, ex);
             }
@@ -484,4 +657,47 @@ public class RollingFileManager extends FileManager {
         }
     }
 
+    private static class EmptyQueue extends ArrayBlockingQueue<Runnable> {
+
+        /**
+         *
+         */
+        private static final long serialVersionUID = 1L;
+
+        EmptyQueue() {
+            super(1);
+        }
+
+        @Override
+        public int remainingCapacity() {
+            return 0;
+        }
+
+        @Override
+        public boolean add(final Runnable runnable) {
+            throw new IllegalStateException("Queue is full");
+        }
+
+        @Override
+        public void put(final Runnable runnable) throws InterruptedException {
+            /* No point in going into a permanent wait */
+            throw new InterruptedException("Unable to insert into queue");
+        }
+
+        @Override
+        public boolean offer(final Runnable runnable, final long timeout, final TimeUnit timeUnit) throws InterruptedException {
+            Thread.sleep(timeUnit.toMillis(timeout));
+            return false;
+        }
+
+        @Override
+        public boolean addAll(final Collection<? extends Runnable> collection) {
+            if (collection.size() > 0) {
+                throw new IllegalArgumentException("Too many items in collection");
+            }
+            return false;
+        }
+
+    }
+
 }
