@@ -26,6 +26,7 @@ import java.util.concurrent.TimeUnit;
 import javax.script.Bindings;
 
 import org.apache.logging.log4j.core.Appender;
+import org.apache.logging.log4j.core.Core;
 import org.apache.logging.log4j.core.Filter;
 import org.apache.logging.log4j.core.LifeCycle2;
 import org.apache.logging.log4j.core.LogEvent;
@@ -36,7 +37,6 @@ import org.apache.logging.log4j.core.config.Configuration;
 import org.apache.logging.log4j.core.config.Node;
 import org.apache.logging.log4j.core.config.plugins.Plugin;
 import org.apache.logging.log4j.core.config.plugins.PluginBuilderFactory;
-import org.apache.logging.log4j.core.config.plugins.PluginConfiguration;
 import org.apache.logging.log4j.core.config.plugins.PluginElement;
 import org.apache.logging.log4j.core.script.AbstractScript;
 import org.apache.logging.log4j.core.script.ScriptManager;
@@ -50,7 +50,7 @@ import org.apache.logging.log4j.core.util.Booleans;
  * the form "$${[key:]token}". The pattern will be resolved each time the Appender is called using
  * the built in StrSubstitutor and the StrLookup plugin that matches the specified key.
  */
-@Plugin(name = "Routing", category = "Core", elementType = Appender.ELEMENT_TYPE, printObject = true)
+@Plugin(name = "Routing", category = Core.CATEGORY_NAME, elementType = Appender.ELEMENT_TYPE, printObject = true)
 public final class RoutingAppender extends AbstractAppender {
 
     public static final String STATIC_VARIABLES_KEY = "staticVariables";
@@ -65,9 +65,6 @@ public final class RoutingAppender extends AbstractAppender {
         @PluginElement("Routes")
         private Routes routes;
 
-        @PluginConfiguration
-        private Configuration configuration;
-
         @PluginElement("RewritePolicy")
         private RewritePolicy rewritePolicy;
 
@@ -86,17 +83,13 @@ public final class RoutingAppender extends AbstractAppender {
                 return null;
             }
             return new RoutingAppender(name, getFilter(), isIgnoreExceptions(), routes, rewritePolicy,
-                    configuration, purgePolicy, defaultRouteScript);
+                    getConfiguration(), purgePolicy, defaultRouteScript);
         }
 
         public Routes getRoutes() {
             return routes;
         }
 
-        public Configuration getConfiguration() {
-            return configuration;
-        }
-
         public AbstractScript getDefaultRouteScript() {
             return defaultRouteScript;
         }
@@ -114,11 +107,6 @@ public final class RoutingAppender extends AbstractAppender {
             return asBuilder();
         }
 
-        public B withConfiguration(@SuppressWarnings("hiding") final Configuration configuration) {
-            this.configuration = configuration;
-            return asBuilder();
-        }
-
         public B withDefaultRouteScript(@SuppressWarnings("hiding") final AbstractScript defaultRouteScript) {
             this.defaultRouteScript = defaultRouteScript;
             return asBuilder();
