@@ -22,24 +22,23 @@ import org.apache.logging.log4j.Level;
 import org.apache.logging.log4j.Marker;
 import org.apache.logging.log4j.ThreadContext;
 import org.apache.logging.log4j.ThreadContext.ContextStack;
+import org.apache.logging.log4j.core.ContextDataInjector;
 import org.apache.logging.log4j.core.Logger;
 import org.apache.logging.log4j.core.LoggerContext;
 import org.apache.logging.log4j.core.config.Configuration;
 import org.apache.logging.log4j.core.config.Property;
 import org.apache.logging.log4j.core.config.ReliabilityStrategy;
 import org.apache.logging.log4j.core.impl.ContextDataFactory;
-import org.apache.logging.log4j.core.ContextDataInjector;
 import org.apache.logging.log4j.core.impl.ContextDataInjectorFactory;
-import org.apache.logging.log4j.core.impl.Log4jLogEvent;
 import org.apache.logging.log4j.core.util.Clock;
 import org.apache.logging.log4j.core.util.ClockFactory;
-import org.apache.logging.log4j.core.util.Constants;
 import org.apache.logging.log4j.core.util.NanoClock;
 import org.apache.logging.log4j.message.Message;
 import org.apache.logging.log4j.message.MessageFactory;
 import org.apache.logging.log4j.message.ReusableMessage;
-import org.apache.logging.log4j.util.StringMap;
 import org.apache.logging.log4j.status.StatusLogger;
+import org.apache.logging.log4j.util.StackLocatorUtil;
+import org.apache.logging.log4j.util.StringMap;
 
 import com.lmax.disruptor.EventTranslatorVararg;
 import com.lmax.disruptor.dsl.Disruptor;
@@ -168,6 +167,13 @@ public class AsyncLogger extends Logger implements EventTranslatorVararg<RingBuf
     }
 
     private void handleRingBufferFull(final RingBufferLogEventTranslator translator) {
+        if (Logger.getRecursionDepth() > 1) { // LOG4J2-1518, LOG4J2-2031
+            // If queue is full AND we are in a recursive call, call appender directly to prevent deadlock
+            final Message message = AsyncQueueFullMessageUtil.transform(translator.message);
+            logMessageInCurrentThread(translator.fqcn, translator.level, translator.marker, message,
+                    translator.thrown);
+            return;
+        }
         final EventRoute eventRoute = loggerDisruptor.getEventRoute(translator.level);
         switch (eventRoute) {
             case ENQUEUE:
@@ -218,7 +224,7 @@ public class AsyncLogger extends Logger implements EventTranslatorVararg<RingBuf
         // location: very expensive operation. LOG4J2-153:
         // Only include if "includeLocation=true" is specified,
         // exclude if not specified or if "false" was specified.
-        return includeLocation ? Log4jLogEvent.calcLocation(fqcn) : null;
+        return includeLocation ? StackLocatorUtil.calcLocation(fqcn) : null;
     }
 
     /**
@@ -243,12 +249,21 @@ public class AsyncLogger extends Logger implements EventTranslatorVararg<RingBuf
             return;
         }
         // if the Message instance is reused, there is no point in freezing its message here
-        if (!Constants.FORMAT_MESSAGES_IN_BACKGROUND && !isReused(message)) { // LOG4J2-898: user may choose
-            message.getFormattedMessage(); // LOG4J2-763: ask message to freeze parameters
+        if (!isReused(message)) {
+            InternalAsyncUtil.makeMessageImmutable(message);
         }
+        StackTraceElement location = null;
         // calls the translateTo method on this AsyncLogger
-        disruptor.getRingBuffer().publishEvent(this, this, calcLocationIfRequested(fqcn), fqcn, level, marker, message,
-                thrown);
+        if (!disruptor.getRingBuffer().tryPublishEvent(this,
+                this, // asyncLogger: 0
+                (location = calcLocationIfRequested(fqcn)), // location: 1
+                fqcn, // 2
+                level, // 3
+                marker, // 4
+                message, // 5
+                thrown)) { // 6
+            handleRingBufferFull(location, fqcn, level, marker, message, thrown);
+        }
     }
 
     /*
@@ -297,6 +312,40 @@ public class AsyncLogger extends Logger implements EventTranslatorVararg<RingBuf
         strategy.log(this, getName(), fqcn, marker, level, message, thrown);
     }
 
+    private void handleRingBufferFull(final StackTraceElement location,
+                                      final String fqcn,
+                                      final Level level,
+                                      final Marker marker,
+                                      final Message msg,
+                                      final Throwable thrown) {
+        if (Logger.getRecursionDepth() > 1) { // LOG4J2-1518, LOG4J2-2031
+            // If queue is full AND we are in a recursive call, call appender directly to prevent deadlock
+            final Message message = AsyncQueueFullMessageUtil.transform(msg);
+            logMessageInCurrentThread(fqcn, level, marker, message, thrown);
+            return;
+        }
+        final EventRoute eventRoute = loggerDisruptor.getEventRoute(level);
+        switch (eventRoute) {
+            case ENQUEUE:
+                loggerDisruptor.getDisruptor().getRingBuffer().publishEvent(this,
+                        this, // asyncLogger: 0
+                        location, // location: 1
+                        fqcn, // 2
+                        level, // 3
+                        marker, // 4
+                        msg, // 5
+                        thrown); // 6
+                break;
+            case SYNCHRONOUS:
+                logMessageInCurrentThread(fqcn, level, marker, msg, thrown);
+                break;
+            case DISCARD:
+                break;
+            default:
+                throw new IllegalStateException("Unknown EventRoute " + eventRoute);
+        }
+    }
+
     /**
      * This method is called by the EventHandler that processes the RingBufferLogEvent in a separate thread.
      * Merges the contents of the configuration map into the contextData, after replacing any variables in the property
@@ -322,7 +371,7 @@ public class AsyncLogger extends Logger implements EventTranslatorVararg<RingBuf
                 final String value = prop.isValueNeedsLookup() //
                         ? privateConfig.config.getStrSubstitutor().replace(event, prop.getValue()) //
                         : prop.getValue();
-                contextData.putValue(prop.getName(), prop.getValue());
+                contextData.putValue(prop.getName(), value);
             }
             event.setContextData(contextData);
         }
