@@ -101,6 +101,11 @@ public abstract class AbstractConfiguration extends AbstractFilterable implement
      */
     protected boolean isShutdownHookEnabled = true;
 
+    /**
+     * Shutdown timeout in milliseconds.
+     */
+    protected long shutdownTimeoutMillis = 0;
+
     /**
      * The Script manager.
      */
@@ -206,7 +211,12 @@ public abstract class AbstractConfiguration extends AbstractFilterable implement
     public void initialize() {
         LOGGER.debug("Initializing configuration {}", this);
         subst.setConfiguration(this);
-        scriptManager = new ScriptManager(this, watchManager);
+        try {
+            scriptManager = new ScriptManager(this, watchManager);
+        } catch (final LinkageError | Exception e) {
+            // LOG4J2-1920 ScriptEngineManager is not available in Android
+            LOGGER.info("Cannot initialize scripting support because this JRE does not support it.", e);
+        }
         pluginManager.collectPlugins(pluginPackages);
         final PluginManager levelPlugins = new PluginManager(Level.CATEGORY);
         levelPlugins.collectPlugins(pluginPackages);
@@ -368,9 +378,9 @@ public abstract class AbstractConfiguration extends AbstractFilterable implement
         root.clearAppenders();
 
         if (watchManager.isStarted()) {
-            watchManager.stop();
+            watchManager.stop(timeout, timeUnit);
         }
-        configurationScheduler.stop();
+        configurationScheduler.stop(timeout, timeUnit);
 
         if (advertiser != null && advertisement != null) {
             advertiser.unadvertise(advertisement);
@@ -395,6 +405,11 @@ public abstract class AbstractConfiguration extends AbstractFilterable implement
         return isShutdownHookEnabled;
     }
 
+    @Override
+    public long getShutdownTimeoutMillis() {
+        return shutdownTimeoutMillis;
+    }
+
     public void setup() {
         // default does nothing, subclasses do work.
     }
@@ -505,8 +520,9 @@ public abstract class AbstractConfiguration extends AbstractFilterable implement
                         LOGGER.error("Script reference to {} not added. Scripts definition cannot contain script references",
                                 script.getName());
                     } else {
-                        scriptManager.addScript(script);
-                    }
+                        if (scriptManager != null) {
+                            scriptManager.addScript(script);
+                        }}
                 }
             } else if (child.getName().equalsIgnoreCase("Appenders")) {
                 appenders = child.getObject();
