@@ -19,11 +19,7 @@ package org.apache.logging.log4j.core.config;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileNotFoundException;
-import java.io.IOException;
-import java.io.InputStream;
-import java.net.MalformedURLException;
 import java.net.URI;
-import java.net.URISyntaxException;
 import java.net.URL;
 import java.util.ArrayList;
 import java.util.Collection;
@@ -43,7 +39,6 @@ import org.apache.logging.log4j.core.config.plugins.util.PluginType;
 import org.apache.logging.log4j.core.lookup.Interpolator;
 import org.apache.logging.log4j.core.lookup.StrSubstitutor;
 import org.apache.logging.log4j.core.util.FileUtils;
-import org.apache.logging.log4j.core.util.Loader;
 import org.apache.logging.log4j.core.util.NetUtils;
 import org.apache.logging.log4j.core.util.ReflectionUtil;
 import org.apache.logging.log4j.status.StatusLogger;
@@ -75,7 +70,7 @@ import org.apache.logging.log4j.util.Strings;
  * last if no configuration has been returned.
  */
 public abstract class ConfigurationFactory extends ConfigurationBuilderFactory {
-    
+
     public ConfigurationFactory() {
         super();
         // TEMP For breakpoints
@@ -239,7 +234,7 @@ public abstract class ConfigurationFactory extends ConfigurationBuilderFactory {
             return null;
         }
         if (configLocation != null) {
-            final ConfigurationSource source = getInputFromUri(configLocation);
+            final ConfigurationSource source = ConfigurationSource.fromUri(configLocation);
             if (source != null) {
                 return getConfiguration(loggerContext, source);
             }
@@ -266,7 +261,7 @@ public abstract class ConfigurationFactory extends ConfigurationBuilderFactory {
         }
         if (isClassLoaderUri(configLocation)) {
             final String path = extractClassLoaderUriPath(configLocation);
-            final ConfigurationSource source = getInputFromResource(path, loader);
+            final ConfigurationSource source = ConfigurationSource.fromResource(path, loader);
             if (source != null) {
                 final Configuration configuration = getConfiguration(loggerContext, source);
                 if (configuration != null) {
@@ -277,43 +272,7 @@ public abstract class ConfigurationFactory extends ConfigurationBuilderFactory {
         return getConfiguration(loggerContext, name, configLocation);
     }
 
-    /**
-     * Loads the configuration from a URI.
-     * @param configLocation A URI representing the location of the configuration.
-     * @return The ConfigurationSource for the configuration.
-     */
-    protected ConfigurationSource getInputFromUri(final URI configLocation) {
-        final File configFile = FileUtils.fileFromUri(configLocation);
-        if (configFile != null && configFile.exists() && configFile.canRead()) {
-            try {
-                return new ConfigurationSource(new FileInputStream(configFile), configFile);
-            } catch (final FileNotFoundException ex) {
-                LOGGER.error("Cannot locate file {}", configLocation.getPath(), ex);
-            }
-        }
-        if (isClassLoaderUri(configLocation)) {
-            final ClassLoader loader = LoaderUtil.getThreadContextClassLoader();
-            final String path = extractClassLoaderUriPath(configLocation);
-            final ConfigurationSource source = getInputFromResource(path, loader);
-            if (source != null) {
-                return source;
-            }
-        }
-        if (!configLocation.isAbsolute()) { // LOG4J2-704 avoid confusing error message thrown by uri.toURL()
-            LOGGER.error("File not found in file system or classpath: {}", configLocation.toString());
-            return null;
-        }
-        try {
-            return new ConfigurationSource(configLocation.toURL().openStream(), configLocation.toURL());
-        } catch (final MalformedURLException ex) {
-            LOGGER.error("Invalid URL {}", configLocation.toString(), ex);
-        } catch (final Exception ex) {
-            LOGGER.error("Unable to access {}", configLocation.toString(), ex);
-        }
-        return null;
-    }
-
-    private static boolean isClassLoaderUri(final URI uri) {
+    static boolean isClassLoaderUri(final URI uri) {
         if (uri == null) {
             return false;
         }
@@ -321,7 +280,7 @@ public abstract class ConfigurationFactory extends ConfigurationBuilderFactory {
         return scheme == null || scheme.equals(CLASS_LOADER_SCHEME) || scheme.equals(CLASS_PATH_SCHEME);
     }
 
-    private static String extractClassLoaderUriPath(final URI uri) {
+    static String extractClassLoaderUriPath(final URI uri) {
         return uri.getScheme() == null ? uri.getPath() : uri.getSchemeSpecificPart();
     }
 
@@ -336,7 +295,7 @@ public abstract class ConfigurationFactory extends ConfigurationBuilderFactory {
             final URL url = new URL(config);
             return new ConfigurationSource(url.openStream(), FileUtils.fileFromUri(url.toURI()));
         } catch (final Exception ex) {
-            final ConfigurationSource source = getInputFromResource(config, loader);
+            final ConfigurationSource source = ConfigurationSource.fromResource(config, loader);
             if (source == null) {
                 try {
                     final File file = new File(config);
@@ -350,39 +309,6 @@ public abstract class ConfigurationFactory extends ConfigurationBuilderFactory {
         }
     }
 
-    /**
-     * Retrieves the configuration via the ClassLoader.
-     * @param resource The resource to load.
-     * @param loader The default ClassLoader to use.
-     * @return The ConfigurationSource for the configuration.
-     */
-    protected ConfigurationSource getInputFromResource(final String resource, final ClassLoader loader) {
-        final URL url = Loader.getResource(resource, loader);
-        if (url == null) {
-            return null;
-        }
-        InputStream is = null;
-        try {
-            is = url.openStream();
-        } catch (final IOException ioe) {
-            LOGGER.catching(Level.DEBUG, ioe);
-            return null;
-        }
-        if (is == null) {
-            return null;
-        }
-
-        if (FileUtils.isFile(url)) {
-            try {
-                return new ConfigurationSource(is, FileUtils.fileFromUri(url.toURI()));
-            } catch (final URISyntaxException ex) {
-                // Just ignore the exception.
-                LOGGER.catching(Level.DEBUG, ex);
-            }
-        }
-        return new ConfigurationSource(is, url);
-    }
-
     /**
      * Default Factory.
      */
@@ -463,14 +389,17 @@ public abstract class ConfigurationFactory extends ConfigurationBuilderFactory {
             if (config != null) {
                 return config;
             }
-            LOGGER.error("No log4j2 configuration file found. Using default configuration: logging only errors to the console.");
+            LOGGER.error("No log4j2 configuration file found. " +
+                    "Using default configuration: logging only errors to the console. " +
+                    "Set system property 'log4j2.debug' " +
+                    "to show Log4j2 internal initialization logging.");
             return new DefaultConfiguration();
         }
 
         private Configuration getConfiguration(final LoggerContext loggerContext, final String configLocationStr) {
             ConfigurationSource source = null;
             try {
-                source = getInputFromUri(NetUtils.toURI(configLocationStr));
+                source = ConfigurationSource.fromUri(NetUtils.toURI(configLocationStr));
             } catch (final Exception ex) {
                 // Ignore the error and try as a String.
                 LOGGER.catching(Level.DEBUG, ex);
@@ -514,8 +443,11 @@ public abstract class ConfigurationFactory extends ConfigurationBuilderFactory {
                     }
                     configName = named ? prefix + name + suffix : prefix + suffix;
 
-                    final ConfigurationSource source = getInputFromResource(configName, loader);
+                    final ConfigurationSource source = ConfigurationSource.fromResource(configName, loader);
                     if (source != null) {
+                        if (!factory.isActive()) {
+                            LOGGER.warn("Found configuration file {} for inactive ConfigurationFactory {}", configName, factory.getClass().getName());
+                        }
                         return factory.getConfiguration(loggerContext, source);
                     }
                 }
