@@ -17,59 +17,81 @@
 package org.apache.logging.log4j.core.config;
 
 import java.util.Date;
+import java.util.Queue;
 import java.util.concurrent.Callable;
 import java.util.concurrent.ScheduledExecutorService;
 import java.util.concurrent.ScheduledFuture;
 import java.util.concurrent.ScheduledThreadPoolExecutor;
 import java.util.concurrent.TimeUnit;
 
+import org.apache.logging.log4j.Logger;
 import org.apache.logging.log4j.core.AbstractLifeCycle;
 import org.apache.logging.log4j.core.util.CronExpression;
 import org.apache.logging.log4j.core.util.Log4jThreadFactory;
+import org.apache.logging.log4j.status.StatusLogger;
 
 /**
  *
  */
 public class ConfigurationScheduler extends AbstractLifeCycle {
 
+    private static final Logger LOGGER = StatusLogger.getLogger();
     private static final String SIMPLE_NAME = "Log4j2 " + ConfigurationScheduler.class.getSimpleName();
     private static final int MAX_SCHEDULED_ITEMS = 5;
+    
     private ScheduledExecutorService executorService;
-
     private int scheduledItems = 0;
+    private final String name;
+
+    public ConfigurationScheduler() {
+        this(SIMPLE_NAME);
+    }
+
+    public ConfigurationScheduler(final String name) {
+        super();
+        this.name = name;
+    }
 
     @Override
     public void start() {
         super.start();
-        if (scheduledItems > 0) {
-            LOGGER.debug("{} starting {} threads", scheduledItems, SIMPLE_NAME);
-            scheduledItems = Math.min(scheduledItems, MAX_SCHEDULED_ITEMS);
-            executorService = new ScheduledThreadPoolExecutor(scheduledItems,
-                    Log4jThreadFactory.createDaemonThreadFactory("Scheduled"));
-        } else {
-            LOGGER.debug("{}: No scheduled items", SIMPLE_NAME);
-        }
     }
 
     @Override
     public boolean stop(final long timeout, final TimeUnit timeUnit) {
         setStopping();
-        if (executorService != null) {
-            LOGGER.debug("{} shutting down threads in {}", SIMPLE_NAME, executorService);
+        if (isExecutorServiceSet()) {
+            LOGGER.debug("{} shutting down threads in {}", name, getExecutorService());
             executorService.shutdown();
+            try {
+                executorService.awaitTermination(timeout, timeUnit);
+            } catch (final InterruptedException ie) {
+                executorService.shutdownNow();
+                try {
+                    executorService.awaitTermination(timeout, timeUnit);
+                } catch (final InterruptedException inner) {
+                    LOGGER.warn("{} stopped but some scheduled services may not have completed.", name);
+                }
+                // Preserve interrupt status
+                Thread.currentThread().interrupt();
+            }
         }
         setStopped();
         return true;
     }
 
+    public boolean isExecutorServiceSet() {
+        return executorService != null;
+    }
+
     /**
      * Increment the number of threads in the pool.
      */
     public void incrementScheduledItems() {
-        if (!isStarted()) {
-            ++scheduledItems;
+        if (isExecutorServiceSet()) {
+            LOGGER.error("{} attempted to increment scheduled items after start", name);
         } else {
-            LOGGER.error("{} attempted to increment scheduled items after start", SIMPLE_NAME);
+            ++scheduledItems;
         }
     }
 
@@ -92,7 +114,7 @@ public class ConfigurationScheduler extends AbstractLifeCycle {
      *
      */
     public <V> ScheduledFuture<V> schedule(final Callable<V> callable, final long delay, final TimeUnit unit) {
-        return executorService.schedule(callable, delay, unit);
+        return getExecutorService().schedule(callable, delay, unit);
     }
 
     /**
@@ -104,7 +126,7 @@ public class ConfigurationScheduler extends AbstractLifeCycle {
      * upon completion.
      */
     public ScheduledFuture<?> schedule(final Runnable command, final long delay, final TimeUnit unit) {
-        return executorService.schedule(command, delay, unit);
+        return getExecutorService().schedule(command, delay, unit);
     }
 
 
@@ -115,11 +137,23 @@ public class ConfigurationScheduler extends AbstractLifeCycle {
      * @return a ScheduledFuture representing the next time the command will run.
      */
     public CronScheduledFuture<?> scheduleWithCron(final CronExpression cronExpression, final Runnable command) {
-        final Date fireDate = cronExpression.getNextValidTimeAfter(new Date());
+        return scheduleWithCron(cronExpression, new Date(), command);
+    }
+
+    /**
+     * Creates and executes an action that first based on a cron expression.
+     * @param cronExpression the cron expression describing the schedule.
+     * @param startDate The time to use as the time to begin the cron expression. Defaults to the current date and time.
+     * @param command The Runnable to run,
+     * @return a ScheduledFuture representing the next time the command will run.
+     */
+    public CronScheduledFuture<?> scheduleWithCron(final CronExpression cronExpression, final Date startDate, final Runnable command) {
+        final Date fireDate = cronExpression.getNextValidTimeAfter(startDate == null ? new Date() : startDate);
         final CronRunnable runnable = new CronRunnable(command, cronExpression);
         final ScheduledFuture<?> future = schedule(runnable, nextFireInterval(fireDate), TimeUnit.MILLISECONDS);
         final CronScheduledFuture<?> cronScheduledFuture = new CronScheduledFuture<>(future, fireDate);
         runnable.setScheduledFuture(cronScheduledFuture);
+        LOGGER.debug("{} scheduled cron expression {} to fire at {}", name, cronExpression.getCronExpression(), fireDate);
         return cronScheduledFuture;
     }
 
@@ -136,7 +170,7 @@ public class ConfigurationScheduler extends AbstractLifeCycle {
      * exception upon cancellation
      */
     public ScheduledFuture<?> scheduleAtFixedRate(final Runnable command, final long initialDelay, final long period, final TimeUnit unit) {
-        return executorService.scheduleAtFixedRate(command, initialDelay, period, unit);
+        return getExecutorService().scheduleAtFixedRate(command, initialDelay, period, unit);
     }
 
     /**
@@ -150,13 +184,31 @@ public class ConfigurationScheduler extends AbstractLifeCycle {
      * exception upon cancellation
      */
     public ScheduledFuture<?> scheduleWithFixedDelay(final Runnable command, final long initialDelay, final long delay, final TimeUnit unit) {
-        return executorService.scheduleWithFixedDelay(command, initialDelay, delay, unit);
+        return getExecutorService().scheduleWithFixedDelay(command, initialDelay, delay, unit);
     }
 
     public long nextFireInterval(final Date fireDate) {
         return fireDate.getTime() - new Date().getTime();
     }
 
+    private ScheduledExecutorService getExecutorService() {
+        if (executorService == null) {
+            if (scheduledItems > 0) {
+                LOGGER.debug("{} starting {} threads", name, scheduledItems);
+                scheduledItems = Math.min(scheduledItems, MAX_SCHEDULED_ITEMS);
+                final ScheduledThreadPoolExecutor executor = new ScheduledThreadPoolExecutor(scheduledItems,
+                        Log4jThreadFactory.createDaemonThreadFactory("Scheduled"));
+                executor.setContinueExistingPeriodicTasksAfterShutdownPolicy(false);
+                executor.setExecuteExistingDelayedTasksAfterShutdownPolicy(false);
+                this.executorService = executor;
+
+            } else {
+                LOGGER.debug("{}: No scheduled items", name);
+            }
+        }
+        return executorService;
+    }
+
     public class CronRunnable implements Runnable {
 
         private final CronExpression cronExpression;
@@ -175,15 +227,51 @@ public class ConfigurationScheduler extends AbstractLifeCycle {
         @Override
         public void run() {
             try {
+                final long millis = scheduledFuture.getFireTime().getTime() - System.currentTimeMillis();
+                if (millis > 0) {
+                    LOGGER.debug("{} Cron thread woke up {} millis early. Sleeping", name, millis);
+                    try {
+                        Thread.sleep(millis);
+                    } catch (final InterruptedException ie) {
+                        // Ignore the interruption.
+                    }
+                }
                 runnable.run();
             } catch(final Throwable ex) {
-                LOGGER.error("{} caught error running command", SIMPLE_NAME, ex);
+                LOGGER.error("{} caught error running command", name, ex);
             } finally {
                 final Date fireDate = cronExpression.getNextValidTimeAfter(new Date());
                 final ScheduledFuture<?> future = schedule(this, nextFireInterval(fireDate), TimeUnit.MILLISECONDS);
+                LOGGER.debug("{} Cron expression {} scheduled to fire again at {}", name, cronExpression.getCronExpression(),
+                        fireDate);
                 scheduledFuture.reset(future, fireDate);
             }
         }
+
+        @Override
+        public String toString() {
+            return "CronRunnable{" + cronExpression.getCronExpression() + " - " + scheduledFuture.getFireTime();
+        }
+    }
+
+    @Override
+    public String toString() {
+        final StringBuilder sb = new StringBuilder("ConfigurationScheduler [name=");
+        sb.append(name);
+        sb.append(", [");
+        if (executorService != null) {
+            final Queue<Runnable> queue = ((ScheduledThreadPoolExecutor) executorService).getQueue();
+            boolean first = true;
+            for (final Runnable runnable : queue) {
+                if (!first) {
+                    sb.append(", ");
+                }
+                sb.append(runnable.toString());
+                first = false;
+            }
+        }
+        sb.append("]");
+        return sb.toString();
     }
 
 }
