@@ -18,7 +18,6 @@ package org.apache.logging.log4j.core.config.plugins.util;
 
 import java.io.File;
 import java.io.FileInputStream;
-import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.io.UnsupportedEncodingException;
 import java.net.URI;
@@ -67,11 +66,11 @@ import org.osgi.framework.wiring.BundleWiring;
  * </p>
  *
  * <pre>
- * ResolverUtil&lt;ActionBean&gt; resolver = new ResolverUtil&lt;ActionBean&gt;();
- * resolver.findImplementation(ActionBean.class, pkg1, pkg2);
+ * ResolverUtil resolver = new ResolverUtil();
+ * resolver.findInPackage(new CustomTest(), pkg1);
  * resolver.find(new CustomTest(), pkg1);
- * resolver.find(new CustomTest(), pkg2);
- * Collection&lt;ActionBean&gt; beans = resolver.getClasses();
+ * resolver.find(new CustomTest(), pkg1, pkg2);
+ * Set&lt;Class&lt;?&gt;&gt; beans = resolver.getClasses();
  * </pre>
  *
  * <p>
@@ -120,7 +119,7 @@ public class ResolverUtil {
     }
 
     /**
-     * Returns the classloader that will be used for scanning for classes. If no explicit ClassLoader has been set by
+     * Returns the ClassLoader that will be used for scanning for classes. If no explicit ClassLoader has been set by
      * the calling, the context class loader will be used.
      *
      * @return the ClassLoader that will be used to scan for classes
@@ -131,7 +130,7 @@ public class ResolverUtil {
 
     /**
      * Sets an explicit ClassLoader that should be used when scanning for classes. If none is set then the context
-     * classloader will be used.
+     * ClassLoader will be used.
      *
      * @param aClassloader
      *        a ClassLoader to use when scanning for classes
@@ -177,7 +176,7 @@ public class ResolverUtil {
         try {
             urls = loader.getResources(packageName);
         } catch (final IOException ioe) {
-            LOGGER.warn("Could not read package: " + packageName, ioe);
+            LOGGER.warn("Could not read package: {}", packageName, ioe);
             return;
         }
 
@@ -186,7 +185,7 @@ public class ResolverUtil {
                 final URL url = urls.nextElement();
                 final String urlPath = extractPath(url);
 
-                LOGGER.info("Scanning for classes in [" + urlPath + "] matching criteria: " + test);
+                LOGGER.info("Scanning for classes in '{}' matching criteria {}", urlPath , test);
                 // Check for a jar in a war in JBoss
                 if (VFSZIP.equals(url.getProtocol())) {
                     final String path = urlPath.substring(0, urlPath.length() - packageName.length() - 2);
@@ -218,7 +217,7 @@ public class ResolverUtil {
                     }
                 }
             } catch (final IOException | URISyntaxException ioe) {
-                LOGGER.warn("could not read entries", ioe);
+                LOGGER.warn("Could not read entries", ioe);
             }
         }
     }
@@ -236,8 +235,9 @@ public class ResolverUtil {
             urlPath = urlPath.substring(5);
         }
         // If it was in a JAR, grab the path to the jar
-        if (urlPath.indexOf('!') > 0) {
-            urlPath = urlPath.substring(0, urlPath.indexOf('!'));
+        final int bangIndex = urlPath.indexOf('!');
+        if (bangIndex > 0) {
+            urlPath = urlPath.substring(0, bangIndex);
         }
 
         // LOG4J2-445
@@ -265,7 +265,7 @@ public class ResolverUtil {
     }
 
     /**
-     * Finds matches in a physical directory on a filesystem. Examines all files within a directory - if the File object
+     * Finds matches in a physical directory on a file system. Examines all files within a directory - if the File object
      * is not a directory, and ends with <i>.class</i> the file is loaded and tested to see if it is acceptable
      * according to the Test. Operates recursively to find classes within a folder structure matching the package
      * structure.
@@ -315,17 +315,13 @@ public class ResolverUtil {
      *        the jar file to be examined for classes
      */
     private void loadImplementationsInJar(final Test test, final String parent, final File jarFile) {
-        @SuppressWarnings("resource")
         JarInputStream jarStream = null;
         try {
             jarStream = new JarInputStream(new FileInputStream(jarFile));
             loadImplementationsInJar(test, parent, jarFile.getPath(), jarStream);
-        } catch (final FileNotFoundException ex) {
-            LOGGER.error("Could not search jar file '" + jarFile + "' for classes matching criteria: " + test
-                    + " file not found", ex);
-        } catch (final IOException ioe) {
-            LOGGER.error("Could not search jar file '" + jarFile + "' for classes matching criteria: " + test
-                    + " due to an IOException", ioe);
+        } catch (final IOException ex) {
+            LOGGER.error("Could not search JAR file '{}' for classes matching criteria {}, file not found", jarFile,
+                    test, ex);
         } finally {
             close(jarStream, jarFile);
         }
@@ -369,8 +365,8 @@ public class ResolverUtil {
                 }
             }
         } catch (final IOException ioe) {
-            LOGGER.error("Could not search jar file '" + path + "' for classes matching criteria: " + test
-                    + " due to an IOException", ioe);
+            LOGGER.error("Could not search JAR file '{}' for classes matching criteria {} due to an IOException", path,
+                    test, ioe);
         }
     }
 
@@ -389,7 +385,7 @@ public class ResolverUtil {
             if (test.doesMatchClass()) {
                 final String externalName = fqn.substring(0, fqn.indexOf('.')).replace('/', '.');
                 if (LOGGER.isDebugEnabled()) {
-                    LOGGER.debug("Checking to see if class " + externalName + " matches criteria [" + test + ']');
+                    LOGGER.debug("Checking to see if class {} matches criteria {}", externalName, test);
                 }
 
                 final Class<?> type = loader.loadClass(externalName);
@@ -407,7 +403,7 @@ public class ResolverUtil {
                 }
             }
         } catch (final Throwable t) {
-            LOGGER.warn("Could not examine class '" + fqn, t);
+            LOGGER.warn("Could not examine class {}", fqn, t);
         }
     }
 
