@@ -24,7 +24,6 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
 import java.util.Map;
-
 import javax.xml.XMLConstants;
 import javax.xml.parsers.DocumentBuilder;
 import javax.xml.parsers.DocumentBuilderFactory;
@@ -120,6 +119,8 @@ public class XmlConfiguration extends AbstractConfiguration implements Reconfigu
                     statusConfig.withDestination(value);
                 } else if ("shutdownHook".equalsIgnoreCase(key)) {
                     isShutdownHookEnabled = !"disable".equalsIgnoreCase(value);
+                } else if ("shutdownTimeout".equalsIgnoreCase(key)) {
+                    shutdownTimeoutMillis = Long.parseLong(value);
                 } else if ("verbose".equalsIgnoreCase(key)) {
                     statusConfig.withVerbosity(value);
                 } else if ("packages".equalsIgnoreCase(key)) {
@@ -189,12 +190,35 @@ public class XmlConfiguration extends AbstractConfiguration implements Reconfigu
     static DocumentBuilder newDocumentBuilder(final boolean xIncludeAware) throws ParserConfigurationException {
         final DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
         factory.setNamespaceAware(true);
+
+        disableDtdProcessing(factory);
+
         if (xIncludeAware) {
             enableXInclude(factory);
         }
         return factory.newDocumentBuilder();
     }
 
+    private static void disableDtdProcessing(final DocumentBuilderFactory factory) throws ParserConfigurationException {
+        factory.setValidating(false);
+        factory.setExpandEntityReferences(false);
+        setFeature(factory, "http://xml.org/sax/features/external-general-entities", false);
+        setFeature(factory, "http://xml.org/sax/features/external-parameter-entities", false);
+        setFeature(factory, "http://apache.org/xml/features/nonvalidating/load-external-dtd", false);
+    }
+    
+    private static void setFeature(final DocumentBuilderFactory factory, final String featureName, final boolean value)
+            throws ParserConfigurationException {
+        try {
+            factory.setFeature(featureName, value);
+        } catch (ParserConfigurationException e) {
+            throw e;
+        } catch (Exception | LinkageError e) {
+            getStatusLogger().error("Caught {} setting feature {} to {} on DocumentBuilderFactory {}: {}",
+                    e.getClass().getCanonicalName(), featureName, value, factory, e, e);
+        }
+    }
+
     /**
      * Enables XInclude for the given DocumentBuilderFactory
      *
