@@ -16,9 +16,11 @@
  */
 package org.apache.logging.log4j.core.impl;
 
+import java.io.IOException;
 import java.io.InvalidObjectException;
 import java.io.ObjectInputStream;
 import java.io.Serializable;
+import java.rmi.MarshalledObject;
 import java.util.List;
 import java.util.Map;
 import java.util.Objects;
@@ -41,6 +43,7 @@ import org.apache.logging.log4j.message.Message;
 import org.apache.logging.log4j.message.ReusableMessage;
 import org.apache.logging.log4j.message.SimpleMessage;
 import org.apache.logging.log4j.message.TimestampMessage;
+import org.apache.logging.log4j.util.StackLocatorUtil;
 import org.apache.logging.log4j.util.StringMap;
 import org.apache.logging.log4j.status.StatusLogger;
 import org.apache.logging.log4j.util.Strings;
@@ -468,6 +471,14 @@ public class Log4jLogEvent implements LogEvent {
         return new Builder(this);
     }
 
+    @Override
+    public Log4jLogEvent toImmutable() {
+        if (getMessage() instanceof ReusableMessage) {
+            makeMessageImmutable();
+        }
+        return this;
+    }
+
     /**
      * Returns the logging Level.
      * @return the Level associated with this event.
@@ -616,27 +627,10 @@ public class Log4jLogEvent implements LogEvent {
         if (loggerFqcn == null || !includeLocation) {
             return null;
         }
-        source = calcLocation(loggerFqcn);
+        source = StackLocatorUtil.calcLocation(loggerFqcn);
         return source;
     }
 
-    public static StackTraceElement calcLocation(final String fqcnOfLogger) {
-        if (fqcnOfLogger == null) {
-            return null;
-        }
-        // LOG4J2-1029 new Throwable().getStackTrace is faster than Thread.currentThread().getStackTrace().
-        final StackTraceElement[] stackTrace = new Throwable().getStackTrace();
-        StackTraceElement last = null;
-        for (int i = stackTrace.length - 1; i > 0; i--) {
-            final String className = stackTrace[i].getClassName();
-            if (fqcnOfLogger.equals(className)) {
-                return last;
-            }
-            last = stackTrace[i];
-        }
-        return null;
-    }
-
     @Override
     public boolean isIncludeLocation() {
         return includeLocation;
@@ -725,13 +719,16 @@ public class Log4jLogEvent implements LogEvent {
         throw new InvalidObjectException("Proxy required");
     }
 
+    public static LogEvent createMemento(final LogEvent logEvent) {
+        return new Log4jLogEvent.Builder(logEvent).build();
+    }
+    
     /**
      * Creates and returns a new immutable copy of this {@code Log4jLogEvent}.
      *
      * @return a new immutable copy of the data in this {@code Log4jLogEvent}
      */
     public static Log4jLogEvent createMemento(final LogEvent event, final boolean includeLocation) {
-        // TODO implement Log4jLogEvent.createMemento()
         return deserialize(serialize(event, includeLocation));
     }
 
@@ -845,7 +842,12 @@ public class Log4jLogEvent implements LogEvent {
         private final Marker marker;
         private final Level level;
         private final String loggerName;
-        private final Message message;
+        // transient since 2.8
+        private final transient Message message;
+        /** since 2.8 */
+        private MarshalledObject<Message> marshalledMessage;
+        /** since 2.8 */
+        private String messageString;
         private final long timeMillis;
         private final transient Throwable thrown;
         private final ThrowableProxy thrownProxy;
@@ -891,10 +893,10 @@ public class Log4jLogEvent implements LogEvent {
             this.level = event.getLevel();
             this.loggerName = event.getLoggerName();
 
-            final Message msg = event.getMessage();
-            this.message = msg instanceof ReusableMessage
-                    ? memento((ReusableMessage) msg)
-                    : msg;
+            final Message temp = event.getMessage();
+            message = temp instanceof ReusableMessage
+                    ? memento((ReusableMessage) temp)
+                    : temp;
             this.timeMillis = event.getTimeMillis();
             this.thrown = event.getThrown();
             this.thrownProxy = event.getThrownProxy();
@@ -919,18 +921,42 @@ public class Log4jLogEvent implements LogEvent {
             return result;
         }
 
+        private static MarshalledObject<Message> marshall(final Message msg) {
+            try {
+                return new MarshalledObject<>(msg);
+            } catch (final Exception ex) {
+                return null;
+            }
+        }
+
+        private void writeObject(final java.io.ObjectOutputStream s) throws IOException {
+            this.messageString = message.getFormattedMessage();
+            this.marshalledMessage = marshall(message);
+            s.defaultWriteObject();
+        }
+
         /**
          * Returns a Log4jLogEvent using the data in the proxy.
          * @return Log4jLogEvent.
          */
         protected Object readResolve() {
-            final Log4jLogEvent result = new Log4jLogEvent(loggerName, marker, loggerFQCN, level, message, thrown,
+            final Log4jLogEvent result = new Log4jLogEvent(loggerName, marker, loggerFQCN, level, message(), thrown,
                     thrownProxy, contextData, contextStack, threadId, threadName, threadPriority, source, timeMillis,
                     nanoTime);
             result.setEndOfBatch(isEndOfBatch);
             result.setIncludeLocation(isLocationRequired);
             return result;
         }
+
+        private Message message() {
+            if (marshalledMessage != null) {
+                try {
+                    return marshalledMessage.get();
+                } catch (final Exception ex) {
+                }
+            }
+            return new SimpleMessage(messageString);
+        }
     }
 
 }
