@@ -24,13 +24,16 @@ import java.util.Map;
 import org.apache.logging.log4j.Level;
 import org.apache.logging.log4j.Marker;
 import org.apache.logging.log4j.ThreadContext;
-import org.apache.logging.log4j.util.ReadOnlyStringMap;
 import org.apache.logging.log4j.core.LogEvent;
+import org.apache.logging.log4j.core.async.InternalAsyncUtil;
 import org.apache.logging.log4j.core.util.Constants;
 import org.apache.logging.log4j.message.Message;
 import org.apache.logging.log4j.message.ParameterizedMessage;
 import org.apache.logging.log4j.message.ReusableMessage;
 import org.apache.logging.log4j.message.SimpleMessage;
+import org.apache.logging.log4j.util.ReadOnlyStringMap;
+import org.apache.logging.log4j.util.StackLocatorUtil;
+import org.apache.logging.log4j.util.StringBuilders;
 import org.apache.logging.log4j.util.StringMap;
 import org.apache.logging.log4j.util.Strings;
 
@@ -72,6 +75,11 @@ public class MutableLogEvent implements LogEvent, ReusableMessage {
         this.parameters = replacementParameters;
     }
 
+    @Override
+    public Log4jLogEvent toImmutable() {
+        return createMemento();
+    }
+
     /**
      * Initialize the fields of this {@code MutableLogEvent} from another event.
      * Similar in purpose and usage as {@link org.apache.logging.log4j.core.impl.Log4jLogEvent.LogEventProxy},
@@ -109,7 +117,6 @@ public class MutableLogEvent implements LogEvent, ReusableMessage {
 
     /**
      * Clears all references this event has to other objects.
-     *
      */
     public void clear() {
         loggerFqcn = null;
@@ -133,7 +140,9 @@ public class MutableLogEvent implements LogEvent, ReusableMessage {
         // where this instance is kept in a ThreadLocal, so it usually does not change.
         // threadName = null; // no need to clear threadName
 
-        trimMessageText();
+        // ensure that excessively long char[] arrays are not kept in memory forever
+        StringBuilders.trimToMaxSize(messageText, Constants.MAX_REUSABLE_MESSAGE_SIZE);
+
         if (parameters != null) {
             for (int i = 0; i < parameters.length; i++) {
                 parameters[i] = null;
@@ -149,14 +158,6 @@ public class MutableLogEvent implements LogEvent, ReusableMessage {
         //nanoTime;
     }
 
-    // ensure that excessively long char[] arrays are not kept in memory forever
-    private void trimMessageText() {
-        if (messageText != null && messageText.length() > Constants.MAX_REUSABLE_MESSAGE_SIZE) {
-            messageText.setLength(Constants.MAX_REUSABLE_MESSAGE_SIZE);
-            messageText.trimToSize();
-        }
-    }
-
     @Override
     public String getLoggerFqcn() {
         return loggerFqcn;
@@ -199,7 +200,7 @@ public class MutableLogEvent implements LogEvent, ReusableMessage {
     @Override
     public Message getMessage() {
         if (message == null) {
-            return (messageText == null) ? EMPTY : this;
+            return messageText == null ? EMPTY : this;
         }
         return message;
     }
@@ -213,11 +214,7 @@ public class MutableLogEvent implements LogEvent, ReusableMessage {
                 parameterCount = reusable.getParameterCount();
             }
         } else {
-            // if the Message instance is reused, there is no point in freezing its message here
-            if (!Constants.FORMAT_MESSAGES_IN_BACKGROUND && msg != null) { // LOG4J2-898: user may choose
-                msg.getFormattedMessage(); // LOG4J2-763: ask message to freeze parameters
-            }
-            this.message = msg;
+            this.message = InternalAsyncUtil.makeMessageImmutable(msg);
         }
     }
 
@@ -344,7 +341,7 @@ public class MutableLogEvent implements LogEvent, ReusableMessage {
         if (loggerFqcn == null || !includeLocation) {
             return null;
         }
-        source = Log4jLogEvent.calcLocation(loggerFqcn);
+        source = StackLocatorUtil.calcLocation(loggerFqcn);
         return source;
     }
 
