@@ -33,11 +33,13 @@ class Log4jJsonModule extends SimpleModule {
     private static final long serialVersionUID = 1L;
     private final boolean encodeThreadContextAsList;
     private final boolean includeStacktrace;
+    private final boolean stacktraceAsString;
 
-    Log4jJsonModule(final boolean encodeThreadContextAsList, final boolean includeStacktrace) {
+    Log4jJsonModule(final boolean encodeThreadContextAsList, final boolean includeStacktrace, final boolean stacktraceAsString) {
         super(Log4jJsonModule.class.getName(), new Version(2, 0, 0, null, null, null));
         this.encodeThreadContextAsList = encodeThreadContextAsList;
         this.includeStacktrace = includeStacktrace;
+        this.stacktraceAsString = stacktraceAsString;
         // MUST init here.
         // Calling this from setupModule is too late!
         //noinspection ThisEscapedInObjectConstruction
@@ -49,9 +51,9 @@ class Log4jJsonModule extends SimpleModule {
         // Calling super is a MUST!
         super.setupModule(context);
         if (encodeThreadContextAsList) {
-            new SetupContextInitializer().setupModule(context, includeStacktrace);
+            new SetupContextInitializer().setupModule(context, includeStacktrace, stacktraceAsString);
         } else {
-            new SetupContextJsonInitializer().setupModule(context, includeStacktrace);
+            new SetupContextJsonInitializer().setupModule(context, includeStacktrace, stacktraceAsString);
         }
     }
 }
