@@ -51,8 +51,11 @@ import org.apache.logging.log4j.core.pattern.ThrowablePatternConverter;
 import org.apache.logging.log4j.core.util.NetUtils;
 import org.apache.logging.log4j.core.util.Patterns;
 import org.apache.logging.log4j.message.Message;
+import org.apache.logging.log4j.message.MessageCollectionMessage;
+import org.apache.logging.log4j.message.StructuredDataCollectionMessage;
 import org.apache.logging.log4j.message.StructuredDataId;
 import org.apache.logging.log4j.message.StructuredDataMessage;
+import org.apache.logging.log4j.util.ProcessIdUtil;
 import org.apache.logging.log4j.util.StringBuilders;
 import org.apache.logging.log4j.util.Strings;
 
@@ -128,14 +131,14 @@ public final class Rfc5424Layout extends AbstractStringLayout {
             final String exceptionPattern, final boolean useTLSMessageFormat, final LoggerFields[] loggerFields) {
         super(charset);
         final PatternParser exceptionParser = createPatternParser(config, ThrowablePatternConverter.class);
-        exceptionFormatters = exceptionPattern == null ? null : exceptionParser.parse(exceptionPattern, false, false);
+        exceptionFormatters = exceptionPattern == null ? null : exceptionParser.parse(exceptionPattern);
         this.facility = facility;
         this.defaultId = id == null ? DEFAULT_ID : id;
         this.enterpriseNumber = ein;
         this.includeMdc = includeMDC;
         this.includeNewLine = includeNL;
         this.escapeNewLine = escapeNL == null ? null : Matcher.quoteReplacement(escapeNL);
-        this.mdcId = mdcId;
+        this.mdcId = id == null ? DEFAULT_MDCID : id;
         this.mdcSdId = new StructuredDataId(mdcId, enterpriseNumber, null, null);
         this.mdcPrefix = mdcPrefix;
         this.eventPrefix = eventPrefix;
@@ -190,28 +193,26 @@ public final class Rfc5424Layout extends AbstractStringLayout {
         final String name = config == null ? null : config.getName();
         configName = Strings.isNotEmpty(name) ? name : null;
         this.fieldFormatters = createFieldFormatters(loggerFields, config);
-        // TODO Java 9: ProcessHandle.current().getPid();
-        this.procId = "-";
+        this.procId = ProcessIdUtil.getProcessId();
     }
 
     private Map<String, FieldFormatter> createFieldFormatters(final LoggerFields[] loggerFields,
             final Configuration config) {
-        final Map<String, FieldFormatter> sdIdMap = new HashMap<>();
-
+        final Map<String, FieldFormatter> sdIdMap = new HashMap<>(loggerFields == null ? 0 : loggerFields.length);
         if (loggerFields != null) {
-            for (final LoggerFields lField : loggerFields) {
-                final StructuredDataId key = lField.getSdId() == null ? mdcSdId : lField.getSdId();
+            for (final LoggerFields loggerField : loggerFields) {
+                final StructuredDataId key = loggerField.getSdId() == null ? mdcSdId : loggerField.getSdId();
                 final Map<String, List<PatternFormatter>> sdParams = new HashMap<>();
-                final Map<String, String> fields = lField.getMap();
+                final Map<String, String> fields = loggerField.getMap();
                 if (!fields.isEmpty()) {
                     final PatternParser fieldParser = createPatternParser(config, null);
 
                     for (final Map.Entry<String, String> entry : fields.entrySet()) {
-                        final List<PatternFormatter> formatters = fieldParser.parse(entry.getValue(), false, false);
+                        final List<PatternFormatter> formatters = fieldParser.parse(entry.getValue());
                         sdParams.put(entry.getKey(), formatters);
                     }
                     final FieldFormatter fieldFormatter = new FieldFormatter(sdParams,
-                            lField.getDiscardIfAllFieldsAreEmpty());
+                            loggerField.getDiscardIfAllFieldsAreEmpty());
                     sdIdMap.put(key.toString(), fieldFormatter);
                 }
             }
@@ -246,7 +247,7 @@ public final class Rfc5424Layout extends AbstractStringLayout {
      * <li>Key: "structured" Value: "true"</li>
      * <li>Key: "format" Value: "RFC5424"</li>
      * </ul>
-     * 
+     *
      * @return Map of content format keys supporting Rfc5424Layout
      */
     @Override
@@ -332,8 +333,8 @@ public final class Rfc5424Layout extends AbstractStringLayout {
     private void appendMessage(final StringBuilder buffer, final LogEvent event) {
         final Message message = event.getMessage();
         // This layout formats StructuredDataMessages instead of delegating to the Message itself.
-        final String text = (message instanceof StructuredDataMessage) ? message.getFormat() : message
-                .getFormattedMessage();
+        final String text = (message instanceof StructuredDataMessage || message instanceof MessageCollectionMessage)
+                ? message.getFormat() : message.getFormattedMessage();
 
         if (text != null && text.length() > 0) {
             buffer.append(' ').append(escapeNewlines(text, escapeNewLine));
@@ -353,7 +354,8 @@ public final class Rfc5424Layout extends AbstractStringLayout {
 
     private void appendStructuredElements(final StringBuilder buffer, final LogEvent event) {
         final Message message = event.getMessage();
-        final boolean isStructured = message instanceof StructuredDataMessage;
+        final boolean isStructured = message instanceof StructuredDataMessage ||
+                message instanceof StructuredDataCollectionMessage;
 
         if (!isStructured && (fieldFormatters != null && fieldFormatters.isEmpty()) && !includeMdc) {
             buffer.append('-');
@@ -361,7 +363,7 @@ public final class Rfc5424Layout extends AbstractStringLayout {
         }
 
         final Map<String, StructuredDataElement> sdElements = new HashMap<>();
-        final Map<String, String> contextMap = event.getContextMap();
+        final Map<String, String> contextMap = event.getContextData().toMap();
 
         if (mdcRequired != null) {
             checkRequired(contextMap);
@@ -382,24 +384,18 @@ public final class Rfc5424Layout extends AbstractStringLayout {
                 union.union(contextMap);
                 sdElements.put(mdcSdIdStr, union);
             } else {
-                final StructuredDataElement formattedContextMap = new StructuredDataElement(contextMap, false);
+                final StructuredDataElement formattedContextMap = new StructuredDataElement(contextMap, mdcPrefix, false);
                 sdElements.put(mdcSdIdStr, formattedContextMap);
             }
         }
 
         if (isStructured) {
-            final StructuredDataMessage data = (StructuredDataMessage) message;
-            final Map<String, String> map = data.getData();
-            final StructuredDataId id = data.getId();
-            final String sdId = getId(id);
-
-            if (sdElements.containsKey(sdId)) {
-                final StructuredDataElement union = sdElements.get(id.toString());
-                union.union(map);
-                sdElements.put(sdId, union);
+            if (message instanceof MessageCollectionMessage) {
+                for (StructuredDataMessage data : ((StructuredDataCollectionMessage)message)) {
+                    addStructuredData(sdElements, data);
+                }
             } else {
-                final StructuredDataElement formattedData = new StructuredDataElement(map, false);
-                sdElements.put(sdId, formattedData);
+                addStructuredData(sdElements, (StructuredDataMessage) message);
             }
         }
 
@@ -409,7 +405,22 @@ public final class Rfc5424Layout extends AbstractStringLayout {
         }
 
         for (final Map.Entry<String, StructuredDataElement> entry : sdElements.entrySet()) {
-            formatStructuredElement(entry.getKey(), mdcPrefix, entry.getValue(), buffer, listChecker);
+            formatStructuredElement(entry.getKey(), entry.getValue(), buffer, listChecker);
+        }
+    }
+
+    private void addStructuredData(final Map<String, StructuredDataElement> sdElements, final StructuredDataMessage data) {
+        final Map<String, String> map = data.getData();
+        final StructuredDataId id = data.getId();
+        final String sdId = getId(id);
+
+        if (sdElements.containsKey(sdId)) {
+            final StructuredDataElement union = sdElements.get(id.toString());
+            union.union(map);
+            sdElements.put(sdId, union);
+        } else {
+            final StructuredDataElement formattedData = new StructuredDataElement(map, eventPrefix, false);
+            sdElements.put(sdId, formattedData);
         }
     }
 
@@ -493,7 +504,7 @@ public final class Rfc5424Layout extends AbstractStringLayout {
         buf.append(Integer.toString(val));
     }
 
-    private void formatStructuredElement(final String id, final String prefix, final StructuredDataElement data,
+    private void formatStructuredElement(final String id, final StructuredDataElement data,
             final StringBuilder sb, final ListChecker checker) {
         if ((id == null && defaultId == null) || data.discard()) {
             return;
@@ -502,9 +513,9 @@ public final class Rfc5424Layout extends AbstractStringLayout {
         sb.append('[');
         sb.append(id);
         if (!mdcSdId.toString().equals(id)) {
-            appendMap(prefix, data.getFields(), sb, noopChecker);
+            appendMap(data.getPrefix(), data.getFields(), sb, noopChecker);
         } else {
-            appendMap(prefix, data.getFields(), sb, checker);
+            appendMap(data.getPrefix(), data.getFields(), sb, checker);
         }
         sb.append(']');
     }
@@ -640,7 +651,7 @@ public final class Rfc5424Layout extends AbstractStringLayout {
             @PluginAttribute(value = "mdcId", defaultString = DEFAULT_MDCID) final String mdcId,
             @PluginAttribute("mdcPrefix") final String mdcPrefix,
             @PluginAttribute("eventPrefix") final String eventPrefix,
-            @PluginAttribute(value = "newLine", defaultBoolean = false) final boolean newLine,
+            @PluginAttribute(value = "newLine") final boolean newLine,
             @PluginAttribute("newLineEscape") final String escapeNL,
             @PluginAttribute("appName") final String appName,
             @PluginAttribute("messageId") final String msgId,
@@ -649,7 +660,7 @@ public final class Rfc5424Layout extends AbstractStringLayout {
             @PluginAttribute("mdcRequired") final String required,
             @PluginAttribute("exceptionPattern") final String exceptionPattern,
             // RFC 5425
-            @PluginAttribute(value = "useTlsMessageFormat", defaultBoolean = false) final boolean useTlsMessageFormat,
+            @PluginAttribute(value = "useTlsMessageFormat") final boolean useTlsMessageFormat,
             @PluginElement("LoggerFields") final LoggerFields[] loggerFields,
             @PluginConfiguration final Configuration config) {
         // @formatter:on
@@ -674,7 +685,7 @@ public final class Rfc5424Layout extends AbstractStringLayout {
         }
 
         public StructuredDataElement format(final LogEvent event) {
-            final Map<String, String> map = new HashMap<>();
+            final Map<String, String> map = new HashMap<>(delegateMap.size());
 
             for (final Map.Entry<String, List<PatternFormatter>> entry : delegateMap.entrySet()) {
                 final StringBuilder buffer = new StringBuilder();
@@ -683,7 +694,7 @@ public final class Rfc5424Layout extends AbstractStringLayout {
                 }
                 map.put(entry.getKey(), buffer.toString());
             }
-            return new StructuredDataElement(map, discardIfEmpty);
+            return new StructuredDataElement(map, eventPrefix, discardIfEmpty);
         }
     }
 
@@ -691,10 +702,13 @@ public final class Rfc5424Layout extends AbstractStringLayout {
 
         private final Map<String, String> fields;
         private final boolean discardIfEmpty;
+        private final String prefix;
 
-        public StructuredDataElement(final Map<String, String> fields, final boolean discardIfEmpty) {
+        public StructuredDataElement(final Map<String, String> fields, final String prefix,
+                                     final boolean discardIfEmpty) {
             this.discardIfEmpty = discardIfEmpty;
             this.fields = fields;
+            this.prefix = prefix;
         }
 
         boolean discard() {
@@ -718,5 +732,13 @@ public final class Rfc5424Layout extends AbstractStringLayout {
         Map<String, String> getFields() {
             return this.fields;
         }
+
+        String getPrefix() {
+            return prefix;
+        }
+    }
+
+    public Facility getFacility() {
+        return facility;
     }
 }
