@@ -16,32 +16,122 @@
  */
 package org.apache.logging.log4j.core.layout;
 
+import java.util.HashMap;
+import java.util.List;
+import java.util.Map;
+
+import javax.script.SimpleBindings;
+
 import org.apache.logging.log4j.Logger;
 import org.apache.logging.log4j.core.LogEvent;
 import org.apache.logging.log4j.core.config.Configuration;
 import org.apache.logging.log4j.core.config.Node;
 import org.apache.logging.log4j.core.config.plugins.Plugin;
-import org.apache.logging.log4j.core.config.plugins.PluginAttribute;
+import org.apache.logging.log4j.core.config.plugins.PluginBuilderAttribute;
+import org.apache.logging.log4j.core.config.plugins.PluginBuilderFactory;
 import org.apache.logging.log4j.core.config.plugins.PluginConfiguration;
 import org.apache.logging.log4j.core.config.plugins.PluginElement;
-import org.apache.logging.log4j.core.config.plugins.PluginFactory;
 import org.apache.logging.log4j.core.pattern.PatternFormatter;
 import org.apache.logging.log4j.core.pattern.PatternParser;
 import org.apache.logging.log4j.core.script.AbstractScript;
 import org.apache.logging.log4j.core.script.ScriptRef;
 import org.apache.logging.log4j.status.StatusLogger;
 
-import javax.script.SimpleBindings;
-import java.util.HashMap;
-import java.util.List;
-import java.util.Map;
-
 /**
  * Selects the pattern to use based on the Marker in the LogEvent.
  */
 @Plugin(name = "ScriptPatternSelector", category = Node.CATEGORY, elementType = PatternSelector.ELEMENT_TYPE, printObject = true)
 public class ScriptPatternSelector implements PatternSelector {
 
+    /**
+     * Custom ScriptPatternSelector builder. Use the {@link #newBuilder() builder factory method} to create this.
+     */
+    public static class Builder implements org.apache.logging.log4j.core.util.Builder<ScriptPatternSelector> {
+
+        @PluginElement("Script") 
+        private AbstractScript script;
+        
+        @PluginElement("PatternMatch") 
+        private PatternMatch[] properties;
+        
+        @PluginBuilderAttribute("defaultPattern") 
+        private String defaultPattern;
+        
+        @PluginBuilderAttribute("alwaysWriteExceptions") 
+        private boolean alwaysWriteExceptions = true;
+        
+        @PluginBuilderAttribute("disableAnsi")
+        private boolean disableAnsi;
+        
+        @PluginBuilderAttribute("noConsoleNoAnsi")
+        private boolean noConsoleNoAnsi;
+
+        @PluginConfiguration
+        private Configuration configuration;
+
+        private Builder() {
+            // nothing
+        }
+
+        @Override
+        public ScriptPatternSelector build() {
+            if (script == null) {
+                LOGGER.error("A Script, ScriptFile or ScriptRef element must be provided for this ScriptFilter");
+                return null;
+            }
+            if (script instanceof ScriptRef) {
+                if (configuration.getScriptManager().getScript(script.getName()) == null) {
+                    LOGGER.error("No script with name {} has been declared.", script.getName());
+                    return null;
+                }
+            }
+            if (defaultPattern == null) {
+                defaultPattern = PatternLayout.DEFAULT_CONVERSION_PATTERN;
+            }
+            if (properties == null || properties.length == 0) {
+                LOGGER.warn("No marker patterns were provided");
+                return null;
+            }
+            return new ScriptPatternSelector(script, properties, defaultPattern, alwaysWriteExceptions, disableAnsi,
+                    noConsoleNoAnsi, configuration);
+        }
+
+        public Builder setScript(final AbstractScript script) {
+            this.script = script;
+            return this;
+        }
+
+        public Builder setProperties(final PatternMatch[] properties) {
+            this.properties = properties;
+            return this;
+        }
+
+        public Builder setDefaultPattern(final String defaultPattern) {
+            this.defaultPattern = defaultPattern;
+            return this;
+        }
+
+        public Builder setAlwaysWriteExceptions(final boolean alwaysWriteExceptions) {
+            this.alwaysWriteExceptions = alwaysWriteExceptions;
+            return this;
+        }
+
+        public Builder setDisableAnsi(final boolean disableAnsi) {
+            this.disableAnsi = disableAnsi;
+            return this;
+        }
+
+        public Builder setNoConsoleNoAnsi(final boolean noConsoleNoAnsi) {
+            this.noConsoleNoAnsi = noConsoleNoAnsi;
+            return this;
+        }
+
+        public Builder setConfiguration(final Configuration config) {
+            this.configuration = config;
+            return this;
+        }
+    }
+    
     private final Map<String, PatternFormatter[]> formatterMap = new HashMap<>();
 
     private final Map<String, String> patternMap = new HashMap<>();
@@ -55,9 +145,13 @@ public class ScriptPatternSelector implements PatternSelector {
     private final Configuration configuration;
 
 
+    /**
+     * @deprecated Use {@link #newBuilder()} instead. This will be private in a future version.
+     */
+    @Deprecated
     public ScriptPatternSelector(final AbstractScript script, final PatternMatch[] properties, final String defaultPattern,
-                                 final boolean alwaysWriteExceptions, final boolean noConsoleNoAnsi,
-                                 final Configuration config) {
+                                 final boolean alwaysWriteExceptions, final boolean disableAnsi,
+                                 final boolean noConsoleNoAnsi, final Configuration config) {
         this.script = script;
         this.configuration = config;
         if (!(script instanceof ScriptRef)) {
@@ -66,7 +160,7 @@ public class ScriptPatternSelector implements PatternSelector {
         final PatternParser parser = PatternLayout.createPatternParser(config);
         for (final PatternMatch property : properties) {
             try {
-                final List<PatternFormatter> list = parser.parse(property.getPattern(), alwaysWriteExceptions, noConsoleNoAnsi);
+                final List<PatternFormatter> list = parser.parse(property.getPattern(), alwaysWriteExceptions, disableAnsi, noConsoleNoAnsi);
                 formatterMap.put(property.getKey(), list.toArray(new PatternFormatter[list.size()]));
                 patternMap.put(property.getKey(), property.getPattern());
             } catch (final RuntimeException ex) {
@@ -74,7 +168,7 @@ public class ScriptPatternSelector implements PatternSelector {
             }
         }
         try {
-            final List<PatternFormatter> list = parser.parse(defaultPattern, alwaysWriteExceptions, noConsoleNoAnsi);
+            final List<PatternFormatter> list = parser.parse(defaultPattern, alwaysWriteExceptions, disableAnsi, noConsoleNoAnsi);
             defaultFormatters = list.toArray(new PatternFormatter[list.size()]);
             this.defaultPattern = defaultPattern;
         } catch (final RuntimeException ex) {
@@ -98,31 +192,44 @@ public class ScriptPatternSelector implements PatternSelector {
     }
 
 
-    @PluginFactory
-    public static ScriptPatternSelector createSelector(@PluginElement("Script") final AbstractScript script,
-                                                       @PluginElement("PatternMatch") final PatternMatch[] properties,
-                                                       @PluginAttribute("defaultPattern") String defaultPattern,
-                                                       @PluginAttribute(value = "alwaysWriteExceptions", defaultBoolean = true) final boolean alwaysWriteExceptions,
-                                                       @PluginAttribute(value = "noConsoleNoAnsi", defaultBoolean = false) final boolean noConsoleNoAnsi,
-                                                       @PluginConfiguration final Configuration config) {
-        if (script == null) {
-            LOGGER.error("A Script, ScriptFile or ScriptRef element must be provided for this ScriptFilter");
-            return null;
-        }
-        if (script instanceof ScriptRef) {
-            if (config.getScriptManager().getScript(script.getName()) == null) {
-                LOGGER.error("No script with name {} has been declared.", script.getName());
-                return null;
-            }
-        }
-        if (defaultPattern == null) {
-            defaultPattern = PatternLayout.DEFAULT_CONVERSION_PATTERN;
-        }
-        if (properties == null || properties.length == 0) {
-            LOGGER.warn("No marker patterns were provided");
-            return null;
-        }
-        return new ScriptPatternSelector(script, properties, defaultPattern, alwaysWriteExceptions, noConsoleNoAnsi, config);
+    /**
+     * Creates a builder for a custom ScriptPatternSelector.
+     *
+     * @return a ScriptPatternSelector builder.
+     */
+    @PluginBuilderFactory
+    public static Builder newBuilder() {
+        return new Builder();
+    }
+
+    /**
+     * Deprecated, use {@link #newBuilder()} instead.
+     * 
+     * @param script
+     * @param properties
+     * @param defaultPattern
+     * @param alwaysWriteExceptions
+     * @param noConsoleNoAnsi
+     * @param configuration
+     * @return a new ScriptPatternSelector
+     * @deprecated Use {@link #newBuilder()} instead.
+     */
+    @Deprecated
+    public static ScriptPatternSelector createSelector(
+            final AbstractScript script,
+            final PatternMatch[] properties,
+            final String defaultPattern,
+            final boolean alwaysWriteExceptions,
+            final boolean noConsoleNoAnsi,
+            final Configuration configuration) {
+        final Builder builder = newBuilder();
+        builder.setScript(script);
+        builder.setProperties(properties);
+        builder.setDefaultPattern(defaultPattern);
+        builder.setAlwaysWriteExceptions(alwaysWriteExceptions);
+        builder.setNoConsoleNoAnsi(noConsoleNoAnsi);
+        builder.setConfiguration(configuration);
+        return builder.build();
     }
 
     @Override
