@@ -25,6 +25,7 @@ import java.io.File;
 import java.io.FileReader;
 
 import org.apache.logging.log4j.Logger;
+import org.apache.logging.log4j.categories.Layouts;
 import org.apache.logging.log4j.core.CoreLoggerContexts;
 import org.apache.logging.log4j.core.selector.ContextSelector;
 import org.apache.logging.log4j.core.selector.CoreContextSelectors;
@@ -32,6 +33,7 @@ import org.apache.logging.log4j.junit.CleanFiles;
 import org.apache.logging.log4j.junit.LoggerContextRule;
 import org.junit.Rule;
 import org.junit.Test;
+import org.junit.experimental.categories.Category;
 import org.junit.rules.RuleChain;
 import org.junit.runner.RunWith;
 import org.junit.runners.Parameterized;
@@ -41,6 +43,7 @@ import org.junit.runners.Parameterized.Parameters;
  * Tests a "complete" XML file a.k.a. a well-formed XML file.
  */
 @RunWith(Parameterized.class)
+@Category(Layouts.Xml.class)
 public class XmlCompleteFileAppenderTest {
 
     public XmlCompleteFileAppenderTest(final Class<ContextSelector> contextSelector) {
