@@ -21,8 +21,11 @@ import java.sql.DriverManager;
 import java.sql.SQLException;
 import java.sql.Statement;
 
+import org.apache.logging.log4j.categories.Appenders;
 import org.apache.logging.log4j.util.Strings;
+import org.junit.experimental.categories.Category;
 
+@Category(Appenders.Jpa.class)
 public class JpaH2AppenderTest extends AbstractJpaAppenderTest {
     private static final String USER_ID = "sa";
     private static final String PASSWORD = Strings.EMPTY;
