@@ -18,12 +18,15 @@
 package org.apache.logging.log4j.core.jackson;
 
 import com.fasterxml.jackson.databind.ObjectMapper;
+import org.apache.logging.log4j.categories.Layouts;
+import org.junit.experimental.categories.Category;
 
+@Category(Layouts.Json.class)
 public class LevelMixInJsonTest extends LevelMixInTest {
 
     @Override
     protected ObjectMapper newObjectMapper() {
-        return new Log4jJsonObjectMapper(false, true);
+        return new Log4jJsonObjectMapper(false, true, false);
     }
 
 }
