@@ -21,12 +21,14 @@ import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertTrue;
 
+import java.nio.charset.Charset;
 import java.nio.charset.StandardCharsets;
 import java.util.List;
 import java.util.Map;
 
 import org.apache.logging.log4j.Level;
 import org.apache.logging.log4j.ThreadContext;
+import org.apache.logging.log4j.categories.Layouts;
 import org.apache.logging.log4j.core.Appender;
 import org.apache.logging.log4j.core.BasicConfigurationFactory;
 import org.apache.logging.log4j.core.Logger;
@@ -35,6 +37,8 @@ import org.apache.logging.log4j.core.config.Configuration;
 import org.apache.logging.log4j.core.config.ConfigurationFactory;
 import org.apache.logging.log4j.core.impl.Log4jLogEvent;
 import org.apache.logging.log4j.core.jackson.Log4jJsonObjectMapper;
+import org.apache.logging.log4j.core.lookup.JavaLookup;
+import org.apache.logging.log4j.core.util.KeyValuePair;
 import org.apache.logging.log4j.message.SimpleMessage;
 import org.apache.logging.log4j.spi.AbstractLogger;
 import org.apache.logging.log4j.test.appender.ListAppender;
@@ -43,10 +47,12 @@ import org.junit.AfterClass;
 import org.junit.Assert;
 import org.junit.BeforeClass;
 import org.junit.Test;
+import org.junit.experimental.categories.Category;
 
 /**
  * Tests the JsonLayout class.
  */
+@Category(Layouts.Json.class)
 public class JsonLayoutTest {
     static ConfigurationFactory cf = new BasicConfigurationFactory();
 
@@ -116,20 +122,30 @@ public class JsonLayoutTest {
         assertFalse(str, str.contains(DQUOTE + name + DQUOTE + propSep));
     }
 
-    private void testAllFeatures(final boolean includeSource, final boolean compact, final boolean eventEol,
-            final boolean includeContext, final boolean contextMapAslist, final boolean includeStacktace)
+    private void testAllFeatures(final boolean locationInfo, final boolean compact, final boolean eventEol,
+            final boolean includeContext, final boolean contextMapAslist, final boolean includeStacktrace)
             throws Exception {
         final Log4jLogEvent expected = LogEventFixtures.createLogEvent();
-        final AbstractJacksonLayout layout = JsonLayout.createLayout(null, includeSource,
-                includeContext, contextMapAslist, false, compact, eventEol, null, null, StandardCharsets.UTF_8, includeStacktace);
+        // @formatter:off
+        final AbstractJacksonLayout layout = JsonLayout.newBuilder()
+                .setLocationInfo(locationInfo)
+                .setProperties(includeContext)
+                .setPropertiesAsList(contextMapAslist)
+                .setComplete(false)
+                .setCompact(compact)
+                .setEventEol(eventEol)
+                .setCharset(StandardCharsets.UTF_8)
+                .setIncludeStacktrace(includeStacktrace)
+                .build();
+        // @formatter:off
         final String str = layout.toSerializable(expected);
         this.toPropertySeparator(compact);
         // Just check for \n since \r might or might not be there.
         assertEquals(str, !compact || eventEol, str.contains("\n"));
-        assertEquals(str, includeSource, str.contains("source"));
+        assertEquals(str, locationInfo, str.contains("source"));
         assertEquals(str, includeContext, str.contains("contextMap"));
-        final Log4jLogEvent actual = new Log4jJsonObjectMapper(contextMapAslist, includeStacktace).readValue(str, Log4jLogEvent.class);
-        LogEventFixtures.assertEqualLogEvents(expected, actual, includeSource, includeContext, includeStacktace);
+        final Log4jLogEvent actual = new Log4jJsonObjectMapper(contextMapAslist, includeStacktrace, false).readValue(str, Log4jLogEvent.class);
+        LogEventFixtures.assertEqualLogEvents(expected, actual, locationInfo, includeContext, includeStacktrace);
         if (includeContext) {
             this.checkMapEntry("MDC.A", "A_Value", compact, str, contextMapAslist);
             this.checkMapEntry("MDC.B", "B_Value", compact, str, contextMapAslist);
@@ -149,7 +165,7 @@ public class JsonLayoutTest {
         this.checkPropertyName("cause", compact, str);
         this.checkPropertyName("commonElementCount", compact, str);
         this.checkPropertyName("localizedMessage", compact, str);
-        if (includeStacktace) {
+        if (includeStacktrace) {
             this.checkPropertyName("extendedStackTrace", compact, str);
             this.checkPropertyName("class", compact, str);
             this.checkPropertyName("method", compact, str);
@@ -170,7 +186,7 @@ public class JsonLayoutTest {
             this.checkPropertyNameAbsent("contextMap", compact, str);
         }
         this.checkPropertyName("contextStack", compact, str);
-        if (includeSource) {
+        if (locationInfo) {
             this.checkPropertyName("source", compact, str);
         } else {
             this.checkPropertyNameAbsent("source", compact, str);
@@ -201,8 +217,18 @@ public class JsonLayoutTest {
         final Configuration configuration = rootLogger.getContext().getConfiguration();
         // set up appender
         final boolean propertiesAsList = false;
-        final AbstractJacksonLayout layout = JsonLayout.createLayout(configuration, true, true, propertiesAsList,
-                true, false, false, null, null, null, true);
+        // @formatter:off
+        final AbstractJacksonLayout layout = JsonLayout.newBuilder()
+                .setConfiguration(configuration)
+                .setLocationInfo(true)
+                .setProperties(true)
+                .setPropertiesAsList(propertiesAsList)
+                .setComplete(true)
+                .setCompact(false)
+                .setEventEol(false)
+                .setIncludeStacktrace(true)
+                .build();
+        // @formatter:on
         final ListAppender appender = new ListAppender("List", null, layout, true, false);
         appender.start();
 
@@ -240,8 +266,20 @@ public class JsonLayoutTest {
         // set up appender
         // Use [[ and ]] to test header and footer (instead of [ and ])
         final boolean propertiesAsList = false;
-        final AbstractJacksonLayout layout = JsonLayout.createLayout(configuration, true, true, propertiesAsList,
-                true, false, false, "[[", "]]", null, true);
+        // @formatter:off
+        final AbstractJacksonLayout layout = JsonLayout.newBuilder()
+                .setConfiguration(configuration)
+                .setLocationInfo(true)
+                .setProperties(true)
+                .setPropertiesAsList(propertiesAsList)
+                .setComplete(true)
+                .setCompact(false)
+                .setEventEol(false)
+                .setHeader("[[".getBytes(Charset.defaultCharset()))
+                .setFooter("]]".getBytes(Charset.defaultCharset()))
+                .setIncludeStacktrace(true)
+                .build();
+        // @formatter:on
         final ListAppender appender = new ListAppender("List", null, layout, true, false);
         appender.start();
 
@@ -281,22 +319,54 @@ public class JsonLayoutTest {
     @Test
     public void testLayoutLoggerName() throws Exception {
         final boolean propertiesAsList = false;
-        final AbstractJacksonLayout layout = JsonLayout.createLayout(null, false, false, propertiesAsList,
-                false, true, false, null, null, StandardCharsets.UTF_8, true);
-        final Log4jLogEvent expected = Log4jLogEvent.newBuilder() //
-                .setLoggerName("a.B") //
-                .setLoggerFqcn("f.q.c.n") //
-                .setLevel(Level.DEBUG) //
-                .setMessage(new SimpleMessage("M")) //
-                .setThreadName("threadName") //
+        // @formatter:off
+        final AbstractJacksonLayout layout = JsonLayout.newBuilder()
+                .setLocationInfo(false)
+                .setProperties(false)
+                .setPropertiesAsList(propertiesAsList)
+                .setComplete(false)
+                .setCompact(true)
+                .setEventEol(false)
+                .setCharset(StandardCharsets.UTF_8)
+                .setIncludeStacktrace(true)
+                .build();
+        // @formatter:on
+        // @formatter:off
+        final Log4jLogEvent expected = Log4jLogEvent.newBuilder()
+                .setLoggerName("a.B")
+                .setLoggerFqcn("f.q.c.n")
+                .setLevel(Level.DEBUG)
+                .setMessage(new SimpleMessage("M"))
+                .setThreadName("threadName")
                 .setTimeMillis(1).build();
+        // @formatter:on
         final String str = layout.toSerializable(expected);
         assertTrue(str, str.contains("\"loggerName\":\"a.B\""));
-        final Log4jLogEvent actual = new Log4jJsonObjectMapper(propertiesAsList, true).readValue(str, Log4jLogEvent.class);
+        final Log4jLogEvent actual = new Log4jJsonObjectMapper(propertiesAsList, true, false).readValue(str, Log4jLogEvent.class);
         assertEquals(expected.getLoggerName(), actual.getLoggerName());
         assertEquals(expected, actual);
     }
 
+    @Test
+    public void testAdditionalFields() throws Exception {
+        final AbstractJacksonLayout layout = JsonLayout.newBuilder()
+                .setLocationInfo(false)
+                .setProperties(false)
+                .setComplete(false)
+                .setCompact(true)
+                .setEventEol(false)
+                .setIncludeStacktrace(false)
+                .setAdditionalFields(new KeyValuePair[] {
+                    new KeyValuePair("KEY1", "VALUE1"),
+                    new KeyValuePair("KEY2", "${java:runtime}"), })
+                .setCharset(StandardCharsets.UTF_8)
+                .setConfiguration(ctx.getConfiguration())
+                .build();
+        final String str = layout.toSerializable(LogEventFixtures.createLogEvent());
+        assertTrue(str, str.contains("\"KEY1\":\"VALUE1\""));
+        assertTrue(str, str.contains("\"KEY2\":\"" + new JavaLookup().getRuntime() + "\""));
+    }
+
     @Test
     public void testLocationOffCompactOffMdcOff() throws Exception {
         this.testAllFeatures(false, false, false, false, false, true);
@@ -322,6 +392,50 @@ public class JsonLayoutTest {
         this.testAllFeatures(false, false, false, false, false, false);
     }
 
+    @Test
+    public void testStacktraceAsString() throws Exception {
+        final String str = prepareJSONForStacktraceTests(true);
+        assertTrue(str, str.contains("\"extendedStackTrace\":\"java.lang.NullPointerException"));
+    }
+
+    @Test
+    public void testStacktraceAsNonString() throws Exception {
+        final String str = prepareJSONForStacktraceTests(false);
+        assertTrue(str, str.contains("\"extendedStackTrace\":["));
+    }
+
+    private String prepareJSONForStacktraceTests(final boolean stacktraceAsString) {
+        final Log4jLogEvent expected = LogEventFixtures.createLogEvent();
+        // @formatter:off
+        final AbstractJacksonLayout layout = JsonLayout.newBuilder()
+                .setCompact(true)
+                .setIncludeStacktrace(true)
+                .setStacktraceAsString(stacktraceAsString)
+                .build();
+        // @formatter:off
+        return layout.toSerializable(expected);
+    }
+
+    @Test
+    public void testIncludeNullDelimiterTrue() throws Exception {
+        final AbstractJacksonLayout layout = JsonLayout.newBuilder()
+                .setCompact(true)
+                .setIncludeNullDelimiter(true)
+                .build();
+        final String str = layout.toSerializable(LogEventFixtures.createLogEvent());
+        assertTrue(str.endsWith("\0"));
+    }
+
+    @Test
+    public void testIncludeNullDelimiterFalse() throws Exception {
+        final AbstractJacksonLayout layout = JsonLayout.newBuilder()
+                .setCompact(true)
+                .setIncludeNullDelimiter(false)
+                .build();
+        final String str = layout.toSerializable(LogEventFixtures.createLogEvent());
+        assertFalse(str.endsWith("\0"));
+    }
+    
     private String toPropertySeparator(final boolean compact) {
         return compact ? ":" : " : ";
     }
